# 2012-07-18 KWS Pure raw SQL code that can be shared across scripts and web apps.
#                The downside is that there is a security risk with this kind of
#                query (SQL injection attack).  However, I got fed up of writing
#                the same query again and again.

#                DO NOT IMPORT ANY DJANGO CODE IN HERE!! Any required Django objects
#                should be passed in by the calling code (e.g. djangoRawObject).

# Detection limits for drawing non-detection arrows on plots.
GLIMIT = 23.6
RLIMIT = 23.6
ILIMIT = 23.6
ZLIMIT = 22.6
YLIMIT = 21.3

# Add new limits for other filters - e.g. w and V.

LIMITS = {"g": GLIMIT,
          "r": RLIMIT,
          "i": ILIMIT,
          "z": ZLIMIT,
          "y": YLIMIT}

# Thresholds for doing PS1 colour plots.
FILTER_RECURRENCE_PERIOD = 0.5 # i.e. 1 day
FILTER_RECURRENCE_PERIOD_GR = 0.5
FILTER_RECURRENCE_PERIOD_RI = 2.0
FILTER_RECURRENCE_PERIOD_IZ = 2.0

# Pure raw text queries.  Note that if used by Django, there must be an associated model
# created in dbviews. The model is passed in as a parameter if required.

# The following 14 filters include PS1 and ATLAS filters
# 2018-11-05 KWS Added the t = tomato filter.
FILTERS = "grizywxBVRIcoht"


LC_POINTS_QUERY_ATLAS = '''\
          select o.id,
                 d.ra,
                 d.`dec`,
                 d.mag,
                 m.mjd_obs mjd,
                 cast(truncate(mjd_obs,3) as char) tdate,
                 m.exptime,
                 m.filter,
                 m.zp,
                 m.expname,
                 m.filename,
                 m.input,
                 m.reference,
                 d.atlas_metadata_id,
                 d.atlas_object_id,
                 o.atlas_designation,
                 o.other_designation,
                 d.tphot_id,
                 d.x,
                 d.y,
                 d.peakval,
                 d.skyval,
                 d.peakfit,
                 d.dpeak,
                 d.skyfit,
                 d.flux,
                 d.dflux,
                 d.major,
                 d.minor,
                 d.phi,
                 d.err,
                 d.chi_N,
                 abs(d.dpeak/d.peakfit) magerr
            from atlas_diff_objects o
            join atlas_diff_detections d
              on (d.atlas_object_id = o.id)
            join atlas_metadata m
              on (d.atlas_metadata_id = m.id)
           where o.id = %s
             and d.peakfit != 0
           '''

# 2015-12-10 KWS Updated query for DDT format schema.
LC_POINTS_QUERY_ATLAS_DDT = '''\
          select o.id,
                 d.ra,
                 d.`dec`,
                 d.mag,
                 m.mjd_obs mjd,
                 cast(truncate(mjd_obs,3) as char) tdate,
                 m.exptime,
                 m.filter,
                 m.zp,
                 m.expname,
                 m.filename,
                 m.input,
                 m.reference,
                 m.object pointing,
                 m.mag5sig,
                 d.atlas_metadata_id,
                 d.atlas_object_id,
                 o.atlas_designation,
                 o.other_designation,
                 d.tphot_id,
                 d.x,
                 d.y,
                 d.peakval,
                 d.skyval,
                 d.peakfit,
                 d.dpeak,
                 d.skyfit,
                 d.flux,
                 d.dflux,
                 d.major,
                 d.minor,
                 d.phi,
                 d.err,
                 d.chi_N,
                 d.dm magerr
            from atlas_diff_objects o
            join atlas_diff_detections d
              on (d.atlas_object_id = o.id)
            join atlas_metadata m
              on (d.atlas_metadata_id = m.id)
           where o.id = %s
             and d.deprecated is null
             and d.mag > 0
           '''

# 2016-03-02 KWS Changed order by mjd_obs desc to just order by mjd_obs
def filterWhereClause(filters = FILTERS):
    """filterWhereClause.

    Args:
        filters:
    """
    whereClauseList = []
    for filter in filters:
        whereClauseList.append('filter = %s')
    whereClause = ' or '.join(whereClauseList)
    orderBy = ' order by mjd_obs'

    return 'and (' + whereClause + ')' + orderBy



ATLAS_METADATA = '''
          select id,
                 mjd_obs mjd,
                 cast(truncate(mjd_obs,3) as char) tdate,
                 exptime,
                 filter,
                 zp,
                 expname,
                 filename,
                 input,
                 reference,
                 object pointing,
                 mag5sig
            from atlas_metadata
          '''

def metadataWhereClause(metadataIds):
    """metadataWhereClause.

    Args:
        metadataIds:
    """
    inClause = ','.join(['%s' for s in metadataIds])

    return 'where id in (' + inClause +') ' 



# 2017-06-16 KWS Updated query for DDC format schema.
LC_POINTS_QUERY_ATLAS_DDC = '''\
          select o.id,
                 d.ra,
                 d.`dec`,
                 d.mag,
                 m.mjd,
                 cast(truncate(mjd,3) as char) tdate,
                 m.texp exptime,
                 m.filt filter,
                 m.magzpt zp,
                 m.obs expname,
                 m.filename,
                 m.input,
                 m.reference,
                 m.obj pointing,
                 m.mag5sig,
                 d.atlas_metadata_id,
                 d.atlas_object_id,
                 o.atlas_designation,
                 o.other_designation,
                 d.det_id tphot_id,
                 d.ra,
                 d.dec,
                 d.mag,
                 d.dmag magerr,
                 d.x,
                 d.y,
                 d.major,
                 d.minor,
                 d.phi,
                 d.det,
                 d.chin,
                 d.pvr,
                 d.ptr,
                 d.pmv,
                 d.pkn,
                 d.pno,
                 d.pbn,
                 d.pcr,
                 d.pxt,
                 d.psc,
                 d.dup,
                 d.wpflx,
                 d.dflx 
            from atlas_diff_objects o
            join atlas_detectionsddc d
              on (d.atlas_object_id = o.id)
            join atlas_metadataddc m
              on (d.atlas_metadata_id = m.id)
           where o.id = %s
             and d.deprecated is null
           '''

# 2016-03-02 KWS Changed order by mjd_obs desc to just order by mjd_obs
def filterWhereClauseddc(filters = FILTERS):
    """filterWhereClauseddc.

    Args:
        filters:
    """
    whereClauseList = []
    for filter in filters:
        whereClauseList.append('filt = %s')
    whereClause = ' or '.join(whereClauseList)
    orderBy = ' order by mjd'

    return 'and (' + whereClause + ')' + orderBy


ATLAS_METADATADDC = '''
          select id,
                 mjd,
                 cast(truncate(mjd,3) as char) tdate,
                 texp exptime,
                 filt filter,
                 magzpt zp,
                 obs expname,
                 filename,
                 input,
                 reference,
                 obj pointing,
                 mag5sig
            from atlas_metadataddc
          '''


# We want to make this query generic, so create as many series as we have Telescopes + Instruments + Filters
# Each telescope name, instrument name and filter is regarded as a separate group for plotting and colour calculations.
# The query extracts ALL data, both LC points and Limits.  The code afterwards will group the series by filter
# ensuring that limits are also represented in the same colour.
FOLLOWUP_PHOTOMETRY_QUERY = """\
          select p.id, p.transient_object_id, p.mjd, p.mag, p.magerr, p.filter, t.name telescope_name, t.description telescope_description, i.name instrument_name, i.description instrument_description
            from tcs_followup_photometry p, tcs_followup_telescopes t, tcs_followup_telescope_instruments i
           where p.telescope_instrument_id = i.id
             and i.telescope_id = t.id
             and p.transient_object_id = %s
        order by telescope_name, instrument_name, p.filter, p.mjd
      """

# Get forced photometry only if the flux s/n ratio > 3 and limit to a minimum MJD.
FORCED_PHOTOMETRY_QUERY = """\
          select id, transient_object_id, mjd_obs, ra_psf, dec_psf, skycell, exptime, psf_inst_mag, psf_inst_mag_sig, cal_psf_mag, psf_inst_flux, psf_inst_flux_sig, filter, zero_pt
            from tcs_forced_photometry
           where transient_object_id = %s
             and mjd_obs > %s
             and psf_inst_flux_sig > 0
             and abs(psf_inst_flux / psf_inst_flux_sig) > 0.0
        order by filter, mjd_obs
      """

# Django just can't do left-joins easily, so we have to hand craft this.
FOLLOWUP_LIST_QUERY = """\
    select o.followup_id rank,
           o.id,
           o.atlas_designation,
           o.other_designation,
           o.ra,
           o.`dec`,
           o.object_classification,
           o.followup_flag_date,
           o.observation_status,
           o.current_trend,
           s.earliest_mjd,
           s.earliest_mag,
           s.earliest_filter,
           s.latest_mjd,
           s.latest_mag,
           s.latest_filter,
           s.catalogue,
           s.catalogue_object_id,
           s.separation,
           o.realbogus_factor,
           s.external_crossmatches,
           s.discovery_target
      from atlas_diff_objects o
 left join tcs_latest_object_stats s
        on (o.id = s.id)
     where followup_id is not null
       and detection_list_id = %s
      """


def followupListQuery(listId, djangoRawObject = None, conn = None):
   """followupListQuery.

   Args:
       listId:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   resultset = []

   if djangoRawObject:
      # Assume a Django query by default
      resultset = djangoRawObject.objects.raw(FOLLOWUP_LIST_QUERY, [listId])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FOLLOWUP_LIST_QUERY, [listId])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  resultset.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return resultset



# In this query, we'll extract each plotting series and their labels in one go.
def followupPhotometryQuery(candidate, djangoRawObject = None, conn = None):
   """followupPhotometryQuery.

   Args:
       candidate:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   resultset = []

   if djangoRawObject:
      # Assume a Django query by default
      resultset = djangoRawObject.objects.raw(FOLLOWUP_PHOTOMETRY_QUERY, [candidate])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FOLLOWUP_PHOTOMETRY_QUERY, [candidate])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  resultset.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return resultset



def getFollowupPhotometry(candidate, djangoRawObject = None, conn = None):
    """Use the query followupPhotometryQuery but organise the data for plotting"""

    # We need to create a dictionary of filters which contain arrays of the data.
    # It's messy to do this from scratch, so defaultdict to the rescue...
    from collections import defaultdict

    recurrences = followupPhotometryQuery(candidate, djangoRawObject = djangoRawObject, conn = conn)

    # Each set of data is returned ordered by telescope, filter, mjd.  So should be easy to create new
    # points arrays.

    fullList = []
    fullListBlanks = []

    # We'll make the filter data a dictionary. The keys will also serve as the point labels.
    filterData = defaultdict(list)
    filterDataBlanks = defaultdict(list)


    for row in recurrences:
        # For the full list, we don't care about which filter or telescope the data came from
        if row.magerr:
            fullList.append([row.mjd, row.mag, row.magerr])
            filterData[row.telescope_name + ' ' + row.filter].append([row.mjd, row.mag, row.magerr])

        # Blank errors mean that this is a limit observation.  This criterion may change in the
        # future, so we'll need to be prepared to alter the code.
        if not row.magerr:
            fullListBlanks.append([row.mjd, row.mag])
            filterDataBlanks[row.telescope_name + ' ' + row.filter].append([row.mjd, row.mag])
 
    # The fullList and fullList blanks data is purely there to allow us to constrain x and y limits
    return filterData, filterDataBlanks, fullList, fullListBlanks


def forcedPhotometryQuery(candidate, djangoRawObject = None, conn = None):
   """This method exists to allow extraction of all the column data for display as raw data"""
   from gkutils.commonutils import Struct

   mjdLimit = 0.0
   resultset = []

   if djangoRawObject:
      # Assume a Django query by default
      resultset = djangoRawObject.objects.raw(FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  resultset.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return resultset


def getForcedPhotometry(candidate, djangoRawObject = None, conn = None, limits = LIMITS):
    """Use the query forcedPhotometryQuery but organise the data for plotting.
       Grab all the flux data while we're at it, so we can do both mags and flux."""

    from collections import defaultdict

    recurrences = forcedPhotometryQuery(candidate, djangoRawObject = djangoRawObject, conn = conn)

    fullList = []
    fullListBlanks = []
    fullListFlux = []

    # We'll make the filter data a dictionary. The keys will also serve as the point labels.
    filterData = defaultdict(list)
    filterDataFlux = defaultdict(list)
    filterDataBlanks = defaultdict(list)


    for row in recurrences:
        fullListFlux.append([row.mjd_obs, row.psf_inst_flux, row.psf_inst_flux_sig])
        filterDataFlux[row.filter].append([row.mjd_obs, row.psf_inst_flux, row.psf_inst_flux_sig])

        # Arbitrarily cut off the cal_psf_mag at 5.0.  Anything brighter than about 10 is likely to be garbage.
        if row.psf_inst_mag_sig and row.cal_psf_mag and row.cal_psf_mag >= 5.0:
            fullList.append([row.mjd_obs, row.cal_psf_mag, row.psf_inst_mag_sig])
            filterData[row.filter].append([row.mjd_obs, row.cal_psf_mag, row.psf_inst_mag_sig])

        # At the moment we don't pick up the forced blanks, but we'll leave this code in
        # for the time being.  We may modify the query in future to feed the blanks.
        if not row.psf_inst_mag_sig or not row.cal_psf_mag or row.cal_psf_mag < 5.0:
            fullListBlanks.append([row.mjd_obs, limits[row.filter]])
            filterDataBlanks[row.filter].append([row.mjd_obs, limits[row.filter]])

    # The fullList and fullList blanks data is purely there to allow us to constrain x and y limits
    return filterData, filterDataBlanks, fullList, fullListBlanks, filterDataFlux, fullListFlux



def lightcurvePlainQuery(candidate, mjdLimit = 55347.0, djangoRawObject = None, conn = None):
   """lightcurvePlainQuery.

   Args:
       candidate:
       mjdLimit:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_PLAIN_TEXT_QUERY, [candidate, mjdLimit, candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_PLAIN_TEXT_QUERY, [candidate, mjdLimit, candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return recurrences


# 2015-10-13 KWS Plain text forced photometry query
def lightcurveForcedPlainQuery(candidate, mjdLimit = 55347.0, djangoRawObject = None, conn = None):
   """lightcurveForcedPlainQuery.

   Args:
       candidate:
       mjdLimit:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return recurrences


# 2013-02-03 KWS Make the query code generic. Pass in the Django object if Django, or the
#                connection object if a standard MySQL query.
# 2015-11-26 KWS Complete rewritten this code to take variable list.  It still dumbly returns
#                a list of filters in positions defined by FILTERS.  In future we'll return a
#                dictionary or class.
def getLightcurvePoints(candidate, filters=FILTERS, lcQuery=LC_POINTS_QUERY_ATLAS_DDT + filterWhereClause(FILTERS), djangoRawObject = None, conn = None):
   """Return mjd, mag, dmag for each filter"""

   # We should return a DICTIONARY or CLASS, not a list of filters.

   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(lcQuery, tuple([candidate] + [f for f in filters]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (lcQuery, tuple([candidate] + [f for f in filters]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   filtersList = []

   # Create a bunch of empty lists for each filter
   for f in FILTERS:
      exec("%s = []" % f)
      exec("%s = []" % ('n' + f))

   fullList = []

   # 2018-10-04 KWS Only plot recurrences with dup >= 0
   #                unless there are none! There must always
   #                be at least one recurrence!
   cleanedRecurrences = []
   for row in recurrences:
      if row.dup >= 0:
          cleanedRecurrences.append(row)

   if len(cleanedRecurrences) > 0:
      recurrences = cleanedRecurrences

   for row in recurrences:
      fullList.append([row.mjd, abs(row.mag), row.magerr])

      if row.filter in FILTERS:
         if row.mag >= 0:
             eval(row.filter).append([row.mjd, row.mag, row.magerr])
         else:
             eval(('n' + row.filter)).append([row.mjd, abs(row.mag), row.magerr])

   for f in FILTERS:
       filtersList.append(eval(f))
   for f in FILTERS:
       filtersList.append(eval(('n' + f)))

   filtersList.append(fullList)

   return filtersList, recurrences


# 2016-05-04 KWS Code to collect non-detection data. Use recurrences as selected above.
def getNonDetections(recurrences, filters = FILTERS, ndQuery=ATLAS_METADATA, catalogueName = 'atlas_diff_detections', searchRadius = 200.0, djangoRawObject = None, conn = None, tolerance = 0.0):
    """Get all the non-detections for an ATLAS object.  We do this
       by cone searching around the object and collecting the unique
       exposures. The exposures in the lightcurve are then removed
       and we now have non-detections.  This is database expensive."""

    recurrenceCoords = [{"RA": row.ra, "DEC": row.dec} for row in recurrences]
    averageObjectCoords, rmsScatter = getRecurrenceData(recurrenceCoords)
    recurrenceDetections = [r.atlas_metadata_id for r in recurrences]

    from gkutils.commonutils import coneSearchHTM, QUICK, FULL, COUNT, CAT_ID_RA_DEC_COLS, Struct

    message, xmObjects = coneSearchHTM(averageObjectCoords['RA'], averageObjectCoords['DEC'], searchRadius, catalogueName, queryType = FULL, conn = conn, django = True)
    metadataIds = []
    uniqueMetadataIds = []
    if xmObjects:
        for xm in xmObjects:
            metadataIds.append(xm[1]['atlas_metadata_id'])
        uniqueMetadataIds = list(set(metadataIds))

    # Now remove the known metadata IDs from this unique list.

    filteredMetadataIds = [x for x in uniqueMetadataIds if x not in recurrenceDetections]
    # ONLY do the query if there was something in the filteredMetadataIds

    if not filteredMetadataIds:
        print("No non-detection data")
        return [[],[],[]]

    #print "UNFILTERED METADATA = ", uniqueMetadataIds
    #print "FILTERED METADATA = ", filteredMetadataIds
    # Now get the actual data from the metadata table.  These are our non-detections.
    # This is a large query containing a lot of IN statements.  We may need to refine how this is done.
    # In very busy star fields this might break the query size limit, in which case we'd need to hit
    # the database multiple times.  We might want to check len(query) before we give it to MySQL.

    refinedNdQuery = ndQuery + metadataWhereClause(filteredMetadataIds) + filterWhereClause(FILTERS)

    #print refinedNdQuery

    blanks = []

    if djangoRawObject:
        # Assume a Django query by default
        blanks = djangoRawObject.objects.raw(refinedNdQuery, tuple(filteredMetadataIds + [f for f in filters]))
    elif conn:
        # Otherwise try a raw MySQL query if we have a connecton object
        import MySQLdb
        try:
             cursor = conn.cursor(MySQLdb.cursors.DictCursor)
             cursor.execute (refinedNdQuery, tuple(filteredMetadataIds + [f for f in filters]))
             results = cursor.fetchall ()
             cursor.close ()
             if results:
                 for result in results:
                     blanks.append(Struct(**result))

        except MySQLdb.Error as e:
            print("Error %d: %s" % (e.args[0], e.args[1]))
            return [[],[],[]]
    else:
        return [[],[],[]]

    filtersList = []

    # Find the last non-detection, if it exists.
    lastNonDetection = None
    firstDetection = recurrences[0]

    # Create a bunch of empty lists for each filter
    for row in FILTERS:
        exec("%s = []" % row)

    fullList = []

    for row in blanks:
        fullList.append([row.mjd, row.mag5sig])
        if row.mjd < firstDetection.mjd - tolerance:
            # This gets set on each iteration, but if mjd >= first detection mjd then
            # this value should contain the last non-detection epoch (or nothing). 
            lastNonDetection = row

        if row.filter in FILTERS:
            eval(row.filter).append([row.mjd, row.mag5sig])

    for f in FILTERS:
        filtersList.append(eval(f))

    filtersList.append(fullList)

    return filtersList, blanks, lastNonDetection

# 2017-11-15 KWS Create a dictionary out of the filters list. This is required
#                in order to pass data to plotly (alternative to flot).
def getFiltersListAsDict(filtersList):
    """getFiltersListAsDict.

    Args:
        filtersList:
    """
    filtersDict = {}

    # The filters list is defined by FILTERS. It's just a list (sometimes empty)
    # for each filter + another list of the combined data.
    for i in range(len(FILTERS)):
        if len(filtersList[i]) > 0:
            if len(filtersList[i][0]) == 3: # we have errors
                filtersDict[FILTERS[i]] = {'mjd': [x[0] for x in filtersList[i]], 'mag': [y[1] for y in filtersList[i]], 'magerr': [e[2] for e in filtersList[i]]}
            else:
                filtersDict[FILTERS[i]] = {'mjd': [x[0] for x in filtersList[i]], 'mag': [y[1] for y in filtersList[i]]}

    # Now add the combined data
    if len(filtersList[-1][0]) == 3:
        filtersDict['all'] = {'mjd': [x[0] for x in filtersList[-1]], 'mag': [y[1] for y in filtersList[-1]], 'magerr': [e[2] for e in filtersList[-1]]}
    else:
        filtersDict['all'] = {'mjd': [x[0] for x in filtersList[-1]], 'mag': [y[1] for y in filtersList[-1]]}

    return filtersDict

# 2017-08-30 KWS New code to collect non-detection data using cone search around the
#                telescope pointing positions.  Use recurrence data to remove existing
#                recurrences.
from gkutils.commonutils import ATLAS_CONESEARCH_RADIUS

def getNonDetectionsUsingATLASFootprint(recurrences, filters = FILTERS, ndQuery=ATLAS_METADATA, catalogueName = 'atlas_metadata', filterWhereClause = filterWhereClause, searchRadius = ATLAS_CONESEARCH_RADIUS, djangoRawObject = None, conn = None, tolerance = 0.0):
    """Get all the non-detections for an ATLAS object.  We do this
       by cone searching around the object and collecting the unique
       exposures. The exposures in the lightcurve are then removed
       and we now have non-detections.  This is database expensive."""

    recurrenceCoords = [{"RA": row.ra, "DEC": row.dec} for row in recurrences]
    averageObjectCoords, rmsScatter = getRecurrenceData(recurrenceCoords)
    recurrenceDetections = [r.atlas_metadata_id for r in recurrences]

    from gkutils.commonutils import coneSearchHTM, QUICK, FULL, COUNT, CAT_ID_RA_DEC_COLS, Struct, isObjectInsideATLASFootprint

    message, xmObjects = coneSearchHTM(averageObjectCoords['RA'], averageObjectCoords['DEC'], searchRadius, catalogueName, queryType = FULL, conn = conn, django = True)
    metadataIds = []
    uniqueMetadataIds = []
    if xmObjects:
        for xm in xmObjects:
            # Eliminate exposures where the object is outside the ATLAS footprint.
            inside = isObjectInsideATLASFootprint(averageObjectCoords['RA'], averageObjectCoords['DEC'], xm[1]['ra'], xm[1]['dec'], separation = xm[0])
            if inside:
                metadataIds.append(xm[1]['id'])
        uniqueMetadataIds = list(set(metadataIds))

    # Now remove the known metadata IDs from this unique list.

    filteredMetadataIds = [x for x in uniqueMetadataIds if x not in recurrenceDetections]
    # ONLY do the query if there was something in the filteredMetadataIds

    if not filteredMetadataIds:
        print("No non-detection data")
        return [[],[],[]]

    refinedNdQuery = ndQuery + metadataWhereClause(filteredMetadataIds) + filterWhereClause(FILTERS)

    blanks = []

    if djangoRawObject:
        # Assume a Django query by default
        blanks = djangoRawObject.objects.raw(refinedNdQuery, tuple(filteredMetadataIds + [f for f in filters]))
    elif conn:
        # Otherwise try a raw MySQL query if we have a connecton object
        import MySQLdb
        try:
             cursor = conn.cursor(MySQLdb.cursors.DictCursor)
             cursor.execute (refinedNdQuery, tuple(filteredMetadataIds + [f for f in filters]))
             results = cursor.fetchall ()
             cursor.close ()
             if results:
                 for result in results:
                     blanks.append(Struct(**result))

        except MySQLdb.Error as e:
            print("Error %d: %s" % (e.args[0], e.args[1]))
            return [[],[],[]]
    else:
        return [[],[],[]]

    filtersList = []

    # Find the last non-detection, if it exists.
    lastNonDetection = None
    firstDetection = recurrences[0]

    # Create a bunch of empty lists for each filter
    for row in FILTERS:
        exec("%s = []" % row)

    fullList = []

    for row in blanks:
        fullList.append([row.mjd, row.mag5sig])
        if row.mjd < firstDetection.mjd - tolerance:
            # This gets set on each iteration, but if mjd >= first detection mjd then
            # this value should contain the last non-detection epoch (or nothing). 
            lastNonDetection = row

        if row.filter in FILTERS:
            eval(row.filter).append([row.mjd, row.mag5sig])

    for f in FILTERS:
        filtersList.append(eval(f))

    filtersList.append(fullList)

    return filtersList, blanks, lastNonDetection


# 2014-11-05 KWS Flux query
def getLightcurvePointsFlux(candidate, filters=FILTERS, applyFudge = False, djangoRawObject = None, conn = None):
   """getLightcurvePointsFlux.

   Args:
       candidate:
       filters:
       applyFudge:
       djangoRawObject:
       conn:
   """

   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_POINTS_FLUX_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_POINTS_FLUX_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      if applyFudge:
         fullList.append(applyFudgeFactor(row))
      else:
         fullList.append([row.mjd, row.flux, row.fluxerr, row.exptime, row.zero_pt])

      # 2013-09-25 KWS Reduced verbose repetative code by using eval.
      if row.filter in FILTERS:
         if applyFudge:
            eval(row.filter).append(applyFudgeFactor(row))
         else:
            eval(row.filter).append([row.mjd, row.flux, row.fluxerr, row.exptime, row.zero_pt])

   return [g, r, i, z, y, w, x, B, V, fullList]



def getLightcurveBlanks(candidate, filters=FILTERS, djangoRawObject = None, conn = None, limits = LIMITS):
   """getLightcurveBlanks.

   Args:
       candidate:
       filters:
       djangoRawObject:
       conn:
       limits:
   """

   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_BLANKS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_BLANKS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      fullList.append([row.mjd])
      if row.filter in FILTERS:
         eval(row.filter).append([float(row.mjd), limits[row.filter]])

   return [g, r, i, z, y, w, x, B, V, fullList]


def getLimitsFromLCData(dataList):
    '''Get the xMax, xMin, yMax, yMin of provided data'''

    # Should probably move this into utils

    # At some stage we also want to provide a mechanism for getting
    # current MJD and current MJD - 90 days...

    import numpy as n

    # This code assumes that all the data is in the form of x, y, error.
    # The dataList elements are ususally the grizy merged array, the
    # grizy blanks and grizy non-detections.

    # If no error is provided, a zero error column will be added.

    # Want to return individual array max/mins in a list, and total max/min

    xMin = []
    yMin = []
    xMax = []
    yMax = []

    firstPass = True

    for row in dataList:

        if not row:
            continue

        dataArray = n.array(row)

        # If there's only ONE column:
        if len(row[0]) < 2:
            nans = []
            for i in range(len(row)):
                nans.append([n.nan])
            dataArray = n.column_stack((dataArray, n.array(nans)) )

        # If there's only TWO columns:
        if len(row[0]) < 3:
            dataArray = n.column_stack((dataArray, n.zeros( (len(row),1) )))

        if firstPass:
            concatArray = dataArray
        else:
            concatArray = n.vstack((dataArray, concatArray))

        firstPass = False

        xMin.append(n.nanmin(dataArray[:,0]))
        xMax.append(n.nanmax(dataArray[:,0]))
        yMin.append(n.nanmin(dataArray[:,1]))
        yMax.append(n.nanmax(dataArray[:,1]))

    if not firstPass:
        # If the code above has run, then firstPass must always be false.
        xMin.append(n.nanmin(concatArray[:,0]))
        xMax.append(n.nanmax(concatArray[:,0]))
        yMin.append(n.nanmin(concatArray[:,1]))
        yMax.append(n.nanmax(concatArray[:,1]))

    # Return 4 arrays whose length is 1+ number of input arrays.
    # The last element of each array is the min/max for the concatenated
    # arrays.

    return xMin, xMax, yMin, yMax


def colourDataPlainQuery(candidate, applyFudge = False, djangoRawObject = None, conn = None):
    """Call the previously defined queries to return plain text colour data"""

    from gkutils.commonutils import getColour, getColourStats

    # All the code to do this query is defined above, so it should be a simple matter
    # of calling that code.

    # Although we request the y-band data, we'll not bother dealing with z-y for the time being.

    lcData, recurrences = getLightcurvePoints(candidate, applyFudge = applyFudge, djangoRawObject = djangoRawObject, conn = conn)

    # For the colour plots (later) list the griz data separately
    g = lcData[0]
    r = lcData[1]
    i = lcData[2]
    z = lcData[3]
    y = lcData[4]

    grColour = []
    riColour = []
    izColour = []

    meangr = None
    meanri = None
    meaniz = None

    grEvolution = None
    riEvolution = None
    izEvolution = None

    if g and r:
        grColour = getColour(g, r, FILTER_RECURRENCE_PERIOD_GR)
        if grColour and len(grColour) > 1:
            meangr, grEvolution = getColourStats(grColour)
    if r and i:
        riColour = getColour(r, i, FILTER_RECURRENCE_PERIOD_RI)
        if riColour and len(riColour) > 1:
            meanri, riEvolution = getColourStats(riColour)
    if i and z:
        izColour = getColour(i, z, FILTER_RECURRENCE_PERIOD_IZ)
        if izColour and len(izColour) > 1:
            meaniz, izEvolution = getColourStats(izColour)

    return grColour, meangr, grEvolution, riColour, meanri, riEvolution, izColour, meaniz, izEvolution


def getRecurrenceData(recurrences):
    """Get all the recurrence plotting data for dynamic recurrence plots"""

    # Create a dict of RA and DEC so that we can call the RMS function.
    # Also, pick out the designated object RA and DEC primary coords.

    from gkutils.commonutils import calculateRMSScatter

    objectCoordinateRecurrences = []
    primaryObjectCoords = {}
    averageObjectCoords = {}

    primaryDetectionFound = False

    avgRa, avgDec, rmsScatter =  calculateRMSScatter(recurrences)
    averageObjectCoords["RA"] = avgRa
    averageObjectCoords["DEC"] = avgDec

    return averageObjectCoords, rmsScatter


def main(argv = None):
    """Test harness"""
    import optparse, sys
    from gkutils.commonutils import dbConnect

    if argv is None:
        argv = sys.argv

    usage = "Usage: %s <username> <password> <database> <hostname> [<candidate>] [<candidate> ...]" % argv[0]
    if len(argv) < 5:
        sys.exit(usage)

    candidateList = []

    parser = optparse.OptionParser(usage=usage)

    parser.add_option('--ddc', default=False, action="store_true",
                      help='Use the DDC schema for queries')


    options, args = parser.parse_args()
    ddc = options.ddc

    username = args[0]
    password = args[1]
    database = args[2]
    hostname = args[3]

    if len(args) > 4:
        for i in range(4,len(args)):
            candidateList.append({'id': int(args[i])})

    conn = dbConnect(hostname, username, password, database)

    #from lightcurvequeries import getLCData

    for candidate in candidateList:
       if ddc:
           p, recurrences = getLightcurvePoints(candidate['id'], lcQuery=LC_POINTS_QUERY_ATLAS_DDC + filterWhereClauseddc(FILTERS), conn = conn)
       else:
           p, recurrences = getLightcurvePoints(candidate['id'], conn = conn)

       if ddc:
           b, blanks, lastNonDetection = getNonDetectionsUsingATLASFootprint(recurrences, conn = conn, ndQuery=ATLAS_METADATADDC, filterWhereClause = filterWhereClauseddc, catalogueName = 'atlas_metadataddc')
       else:
           b, blanks, lastNonDetection = getNonDetectionsUsingATLASFootprint(recurrences, conn = conn)
           #b, blanks, lastNonDetection = getNonDetections(recurrences, conn = conn, searchRadius=500, tolerance = 0.001)

       #print p, recurrences

       print("DETECTIONS")
       exposures = []
       print("mjd filter expname pointing mag5sig diffimage tphotfile")
       for row in recurrences:
           aux = ''
           if int(row.mjd) >= 57466:
               aux = 'AUX/'
           print(row.mjd, row.filter, row.expname, row.pointing, row.mag5sig, '/atlas/diff/02a/' + str(int(row.mjd)) + '/' + row.expname + '.diff.fz', '/atlas/red/02a/' + str(int(row.mjd)) + '/' + aux + row.expname + '.tph')
           exposures.append(row.expname)


       #print b
       print("NON-DETECTIONS")
       print("mjd filter expname pointing mag5sig diffimage tphotfile")
       for row in blanks:
           aux = ''
           if int(row.mjd) >= 57466:
               aux = 'AUX/'
           print(row.mjd, row.filter, row.expname, row.pointing, row.mag5sig, '/atlas/diff/02a/' + str(int(row.mjd)) + '/' + row.expname + '.diff.fz', '/atlas/red/02a/' + str(int(row.mjd)) + '/' + aux + row.expname + '.tph')
           exposures.append(row.expname)

       if lastNonDetection:
           print("LAST NON-DETECTION = ", lastNonDetection.mjd, lastNonDetection.filter, lastNonDetection.expname, lastNonDetection.pointing, lastNonDetection.mag5sig, lastNonDetection.exptime)

       uniqueExposures = sorted(list(set(exposures)))
       exposureList = []
       tphotList = []

       for exp in uniqueExposures:
           camera = exp[0:3]
           mjd = exp[3:8]
           diffImage = camera + '/' + mjd + '/' + exp + '.diff.fz'
           aux = ''
           if int(mjd) >= 57466:
               aux = 'AUX/'
           inputTphot = camera + '/' + mjd + '/' + aux + exp + '.tph'
           exposureList.append(diffImage)
           tphotList.append(inputTphot)


       for row in tphotList:
           print(row)

       print()

       for row in exposureList:
           print(row)
       #lcPointsJSON, plotLabelsJSON, plotLimits = getLCData(candidate['id'], conn = conn)
       #print lcPointsJSON
       #print plotLabelsJSON
       #print plotLimits

if __name__ == '__main__':
    main()



import os
import sys

from django.core.wsgi import get_wsgi_application

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(BASE_DIR)
os.environ['DJANGO_SETTINGS_MODULE'] = 'atlas.settings'
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'atlas.settings')
application = get_wsgi_application()



from django.db import models


# Special Sloan Custom model for crossmatches ...
# No Meta class associated with this class.

class SloanCrossMatch(models.Model):
    """SloanCrossMatch.
    """

    id = models.IntegerField(primary_key=True)
    catalogue_object_id = models.IntegerField(null=True, blank=True)
    description = models.CharField(max_length=60)
    Objid = models.BigIntegerField() # Field name made lowercase.
    type = models.IntegerField(null=True, blank=True)
    g_ = models.FloatField(null=True, blank=True)
    r_ = models.FloatField(null=True, blank=True)
    i_ = models.FloatField(null=True, blank=True)
    z_ = models.FloatField(null=True, blank=True)
    separation = models.FloatField(null=True, blank=True)
    z = models.FloatField(null=True, blank=True)
    scale = models.FloatField(null=True, blank=True)
    distance = models.FloatField(null=True, blank=True)
    distance_modulus = models.FloatField(null=True, blank=True)
    association_type = models.IntegerField(null=True, blank=True)


# ... and Custom query for FGSS

# 2012-03-05 KWS Modified query so that the only table matched in the crossmatches
#                table is tcs_cat_sdss_stars_galaxies.  This is because we can now
#                have Milliquas matches here.
def getSloanCrossmatchData(transientObjectId):
    """getSloanCrossmatchData.

    Args:
        transientObjectId:
    """
    queryset = SloanCrossMatch.objects.raw('''
           select c.id,
                  c.catalogue_object_id,
                  t.description,
                  cat.Objid,
                  cat.type,
                  cat.g g_, cat.r r_, cat.i i_, cat.z z_,
                  c.separation,
                  c.z,
                  c.scale,
                  c.distance,
                  c.distance_modulus,
                  c.association_type
             from tcs_cross_matches c,
                  psdb_web_v_cat_sdss_stars_galaxies cat,
                  tcs_catalogue_tables t
            where c.transient_object_id = %s
              and cast(c.catalogue_object_id as unsigned) = cat.Objid
              and c.catalogue_table_id = t.id
              and t.table_name = 'tcs_cat_sdss_stars_galaxies'
         order by c.association_type, c.separation
        ''', [transientObjectId])

    return queryset






class PsdbWebVCatSdssStarsGalaxies(models.Model):
    """PsdbWebVCatSdssStarsGalaxies.
    """

    objid = models.BigIntegerField(primary_key=True, db_column='Objid') # Field name made lowercase.
    ra = models.FloatField()
    dec_ = models.FloatField()
    mode = models.IntegerField(null=True, blank=True)
    type = models.IntegerField(null=True, blank=True)
    specobjid = models.BigIntegerField(null=True, db_column='specObjID', blank=True) # Field name made lowercase.
    petromag_u = models.FloatField(null=True, db_column='petroMag_u', blank=True) # Field name made lowercase.
    petromag_g = models.FloatField(null=True, db_column='petroMag_g', blank=True) # Field name made lowercase.
    petromag_r = models.FloatField(null=True, db_column='petroMag_r', blank=True) # Field name made lowercase.
    petromag_i = models.FloatField(null=True, db_column='petroMag_i', blank=True) # Field name made lowercase.
    petromag_z = models.FloatField(null=True, db_column='petroMag_z', blank=True) # Field name made lowercase.
    extinction_u = models.FloatField(null=True, blank=True)
    extinction_g = models.FloatField(null=True, blank=True)
    extinction_r = models.FloatField(null=True, blank=True)
    extinction_i = models.FloatField(null=True, blank=True)
    extinction_z = models.FloatField(null=True, blank=True)
    petromagerr_u = models.FloatField(null=True, db_column='petroMagErr_u', blank=True) # Field name made lowercase.
    petromagerr_g = models.FloatField(null=True, db_column='petroMagErr_g', blank=True) # Field name made lowercase.
    petromagerr_r = models.FloatField(null=True, db_column='petroMagErr_r', blank=True) # Field name made lowercase.
    petromagerr_i = models.FloatField(null=True, db_column='petroMagErr_i', blank=True) # Field name made lowercase.
    petromagerr_z = models.FloatField(null=True, db_column='petroMagErr_z', blank=True) # Field name made lowercase.
    z_ = models.FloatField(null=True, blank=True)
    zerr = models.FloatField(null=True, db_column='zErr', blank=True) # Field name made lowercase.
    dmod = models.FloatField(null=True, blank=True)
    rest_ug = models.FloatField(null=True, blank=True)
    rest_gr = models.FloatField(null=True, blank=True)
    rest_ri = models.FloatField(null=True, blank=True)
    rest_iz = models.FloatField(null=True, blank=True)
    absmag_u = models.FloatField(null=True, db_column='absMag_u', blank=True) # Field name made lowercase.
    absmag_g = models.FloatField(null=True, db_column='absMag_g', blank=True) # Field name made lowercase.
    absmag_r = models.FloatField(null=True, db_column='absMag_r', blank=True) # Field name made lowercase.
    absmag_i = models.FloatField(null=True, db_column='absMag_i', blank=True) # Field name made lowercase.
    absmag_z = models.FloatField(null=True, db_column='absMag_z', blank=True) # Field name made lowercase.
    u = models.FloatField(null=True, blank=True)
    g = models.FloatField(null=True, blank=True)
    r = models.FloatField(null=True, blank=True)
    i = models.FloatField(null=True, blank=True)
    z = models.FloatField(null=True, blank=True)
    err_u = models.FloatField(null=True, db_column='Err_u', blank=True) # Field name made lowercase.
    err_g = models.FloatField(null=True, db_column='Err_g', blank=True) # Field name made lowercase.
    err_r = models.FloatField(null=True, db_column='Err_r', blank=True) # Field name made lowercase.
    err_i = models.FloatField(null=True, db_column='Err_i', blank=True) # Field name made lowercase.
    err_z = models.FloatField(null=True, db_column='Err_z', blank=True) # Field name made lowercase.
    psfmag_u = models.FloatField(null=True, db_column='psfMag_u', blank=True) # Field name made lowercase.
    psfmag_g = models.FloatField(null=True, db_column='psfMag_g', blank=True) # Field name made lowercase.
    psfmag_r = models.FloatField(null=True, db_column='psfMag_r', blank=True) # Field name made lowercase.
    psfmag_i = models.FloatField(null=True, db_column='psfMag_i', blank=True) # Field name made lowercase.
    psfmag_z = models.FloatField(null=True, db_column='psfMag_z', blank=True) # Field name made lowercase.
    psfmagerr_u = models.FloatField(null=True, db_column='psfMagErr_u', blank=True) # Field name made lowercase.
    psfmagerr_g = models.FloatField(null=True, db_column='psfMagErr_g', blank=True) # Field name made lowercase.
    psfmagerr_r = models.FloatField(null=True, db_column='psfMagErr_r', blank=True) # Field name made lowercase.
    psfmagerr_i = models.FloatField(null=True, db_column='psfMagErr_i', blank=True) # Field name made lowercase.
    psfmagerr_z = models.FloatField(null=True, db_column='psfMagErr_z', blank=True) # Field name made lowercase.
    htm20id = models.BigIntegerField(null=True, db_column='htm20ID', blank=True) # Field name made lowercase.
    htm16id = models.BigIntegerField(null=True, db_column='htm16ID', blank=True) # Field name made lowercase.
    cx = models.FloatField(null=True, blank=True)
    cy = models.FloatField(null=True, blank=True)
    cz = models.FloatField(null=True, blank=True)
    class Meta:
        """Meta.
        """

        db_table = 'psdb_web_v_cat_sdss_stars_galaxies'


# Form display options file

PROMOTION_CHOICES = (
    ('Confirmed', 'Promote Object to Confirmed List'),
    ('Good', 'Promote Object to Good List'),
    ('Attic', 'Promote Object to Attic List'),
    ('Eyeball', 'Move Object to Eyeball List'),
    ('Possible', 'Promote Object to Possible List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Undecided - Comment only'),
)

# This is a bit dumb.  We will need to revise how we do these lists.

# It's an EYEBALL object
EYEBALL_PROMOTION_CHOICES = (
    ('Good', 'Promote Object to Good List'),
    ('Attic', 'Promote Object to Attic List'),
    ('Possible', 'Promote Object to Possible List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Undecided - Comment only'),
)

# It's a GOOD object
GOOD_POST_PROMOTION_CHOICES = (
    ('Confirmed', 'Promote Object to Confirmed List'),
    ('Attic', 'Move Object to Attic List'),
    ('Possible', 'Move Object to Possible List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Add a Comment'),
)

# 2019-07-08 KWS Removed "good" choice.
# It's a POSSIBLE object
POSSIBLE_POST_PROMOTION_CHOICES = (
    ('Good', 'Promote Object to Good List'),
    ('Attic', 'Move Object to Attic List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Add a Comment'),
)

# It's an ATTIC object
# 2019-07-08 KWS Removed "good" choice.
ATTIC_POST_PROMOTION_CHOICES = (
    ('Good', 'Promote Object to Good List'),
    ('Possible', 'Move Object to Possible List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Add a Comment'),
)

# It's a STAR object
STAR_POST_PROMOTION_CHOICES = (
    ('Good', 'Promote Object to Good List'),
    ('Attic', 'Move Object to Attic List'),
    ('Possible', 'Move Object to Possible List'),
    ('Eyeball', 'Move Object to Eyeball List'),
    ('Garbage', 'Consign to Garbage'),
    ('DoNothing', 'Add a Comment'),
)

# It's a CONFIRMED object
CONFIRMED_POST_PROMOTION_CHOICES = (
    ('Good', 'Move Object to Good List'),
    ('Attic', 'Move Object to Attic List'),
    ('Possible', 'Move Object to Possible List'),
    ('DoNothing', 'Add a Comment'),
)

# 2011-10-04 KWS Need to be able to comment on garbage or return to eyeball
# It's in the GARBAGE.  Maybe we want to comment or promote back to EYEBALL.
GARBAGE_CHOICES = (
    ('Eyeball', 'Move Object to Eyeball List'),
    ('DoNothing', 'Add a Comment'),
)


# All the possible options for the object's observation status
OBSERVATION_STATUS_CHOICES = (
    (None, None),
    ('in Queue', 'in Queue'),
    ('in Queue NTT', 'in Queue NTT'),
    ('in Queue UH88', 'in Queue UH88'),
    ('in Queue LT', 'in Queue LT'),
    ('in Queue SOAR', 'in Queue SOAR'),
    ('in Queue Gemini N', 'in Queue Gemini N'),
    ('in Queue Gemini S', 'in Queue Gemini S'),
    ('in Queue WHT', 'in Queue WHT'),
    ('in Queue INT', 'in Queue INT'),
    ('in Queue Magellen', 'in Queue Magellan'),
    ('in Queue MMT', 'in Queue MMT'),
    ('in Queue NOT', 'in Queue NOT'),
    ('in Queue VLT', 'in Queue VLT'),
    ('in Queue APO', 'in Queue APO'),
    ('Observed', 'Observed'),
    ('AGN', 'AGN'),
    ('QSO', 'QSO'),
    ('Galaxy', 'Galaxy'),
    ('TDE', 'TDE'),
    ('CV', 'CV'),
    ('LBV', 'LBV'),
    ('LRN', 'LRN'),
    ('Varstar', 'Varstar'),
    ('M dwarf', 'M dwarf'),
    ('ILRT', 'ILRT'),
    ('Nova', 'Nova'),
    ('SN', 'SN'), # It's a SN but we don't yet know what type
    ('SN I', 'SN I'),
    ('SN I-faint', 'SN I-faint'),
    ('SN I-rapid', 'SN I-rapid'),
    ('SN Ia', 'SN Ia'),
    ('SN Ia-91bg-like', 'SN Ia-91bg-like'),
    ('SN Ia-91T-like', 'SN Ia-91T-like'),
    ('SN Ia-pec', 'SN Ia-pec'),
    ('SN Ia-CSM', 'SN Ia-CSM'),
    ('SN Ia-Ca-rich', 'SN Ia-Ca-rich'),
    ('SN Ia-SC', 'SN Ia-SC'),
    ('SN Iax[02cx-like]', 'SN Iax[02cx-like]'),
    ('SN Ib', 'SN Ib'),
    ('SN Ib-Ca-rich', 'SN Ib-Ca-rich'),
    ('SN Ib-pec', 'SN Ib-pec'),
    ('SN Ib/c', 'SN Ib/c'),
    ('SN Ib/c-pec', 'SN Ib/c-pec'),
    ('SN Ibn', 'SN Ibn'),
    ('SN Ic', 'SN Ic'),
    ('SN Ic-BL', 'SN Ic-BL'),
    ('SN Ic-pec', 'SN Ic-pec'),
    ('SN II', 'SN II'),
    ('SN II-pec', 'SN II-pec'),
    ('SN IIb', 'SN IIb'),
    ('SN IIL', 'SN IIL'),
    ('SN IIn', 'SN IIn'),
    ('SN IIn-pec', 'SN IIn-pec'),
    ('SN IIn / AGN', 'SN IIn / AGN'),
    ('SN IIP', 'SN IIP'),
    ('SN II 87A-like', 'SN II 87A-like'),
    ('Kilonova', 'Kilonova'),
    ('SLSN', 'SLSN'),
    ('SLSN-I', 'SLSN-I'),
    ('SLSN-II', 'SLSN-II'),
    ('SLSN-R', 'SLSN-R'),
    ('Impostor-SN', 'Impostor-SN'),
    ('mover', 'mover'),
    ('other', 'other'),
)

#WHOLE_SURVEY_LIFETIME        = 1
#JUST_RECENT_90_DAYS          = 2
#JUST_DETECTION_LIMITS        = 3
#USER_DEFINED_LIMITS          = 4

# 2011-08-30 KWS Added choices for redrawing light curves
LIGHTCURVE_REPLOTTING_CHOICES = (
    ('0', 'Do nothing'),
    ('1', 'Whole Survey Lifetime'),
    ('2', 'Most Recent 90 Days'),
    ('3', 'Just Detection Limits'),
    ('4', 'User Defined Date Limits'),
)

# 2021-05-12 KWS Added selector templates for Django Tables2 for quickview.

choiceSelectors = {'T': '0', 'C': '1', 'G': '2', 'P': '3', 'E': '4', 'A': '5', 'U': '1000'}

def getChoiceSelectorTemplate(choice, checked = ''):
    templateParameters = {'template': None, 'attrs': None}
    try:
        CHOICE_SELECTOR = '''
            <div class="radio-ugpt-%s">
              <input class="%s" type="radio" id="{{record.id}}_promote_demote_%s" value="%s" name="{{record.id}}_promote_demote" %s />
              <label for="{{record.id}}_promote_demote_%s">%s</label>
            </div>
        ''' % (choice.lower(), choice, choiceSelectors[choice], choice, checked, choiceSelectors[choice], choice)
        attrs={"th":{"id":"select_all_%s" % (choice.lower()), "name":"select_all_%s" % (choice.lower()), "value":"all_%s" % (choice.lower())}}
    except KeyError as e:
        CHOICE_SELECTOR = None
        attrs = None

    templateParameters['template'] = CHOICE_SELECTOR
    templateParameters['attrs'] = attrs
    return templateParameters




# Functions within this file are too Django-specific to include in 'utils'.

# PS1 version of the Django Search Form Helper Function.

from psdb.models import TcsTransientObjects
from psdb.models import TcsImages, TcsLatestObjectStats
from gkutils.commonutils import getObjectNamePortion, getCoordsAndSearchRadius, coneSearchHTM, FULL, Struct, ra_to_sex, dec_to_sex
from django.db.models import Q    # Need Q objects for OR query
import sys

SHOW_LC_DATA_LIMIT = 50


def processSearchForm(searchText, getAssociatedData = False, getNonDets = False, getNearbyObjects = False):
    """processSearchForm.

    Args:
        searchText:
        getAssociatedData:
        getNonDets:
        getNearbyObjects:
    """
    from django.db import connection

    results = []
    # Is it a name?
    name = getObjectNamePortion(searchText)
    if name:
        #name = name.replace(' ','')
        # Do a name search
        if name.startswith('20'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('PS'):
            results = TcsTransientObjects.objects.filter(Q(ps1_designation__isnull = False) & Q(ps1_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('AT'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('AT','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('SN'):
            results = TcsTransientObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('SN','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        else: # name.startswith('ZTF'):
            # Check for external crossmatches.
            q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
            results = list(set([x.id for x in q]))
    else:
         # It must be a coordinate or a match failure
         coords = getCoordsAndSearchRadius(searchText)
         if coords:
             searchRadius = 4.0
             if coords['radius']:
                 searchRadius = float(coords['radius'])
                 if searchRadius > 99.0:
                      searchRadius = 99.0

             message, xmObjects = coneSearchHTM(coords['ra'], coords['dec'], searchRadius, 'tcs_transient_objects', queryType = FULL, conn = connection, django = True)
             for xm in xmObjects:
                 dictRow = xm[1]
                 dictRow.update({'xmseparation': xm[0]})
                 # Add sexagesimal RA and Dec
                 dictRow.update({'ra_sex': ra_to_sex(dictRow['ra_psf'])})
                 dictRow.update({'dec_sex': dec_to_sex(dictRow['dec_psf'])})
                 if dictRow['tcs_images_id']:
                     # Get images
                     images = TcsImages.objects.get(pk=dictRow['tcs_images_id'])
                     if images:
                         dictRow.update({'tcs_images_id': images})
                 # 2018-08-07 KWS Convert the class into a dict so we can use common code elsewhere.
                 results.append(Struct(**dictRow))

    return results


def getNearbyObjectsForScatterPlot(candidate, ra, dec, coneSearchRadius = 8.0):
    """getNearbyObjectsForScatterPlot.

    Args:
        candidate:
        ra:
        dec:
        coneSearchRadius:
    """
    from django.db import connection

    recurrencePlotData = []
    recurrencePlotLabels = []
    averageObjectCoords = []
    rmsScatter = []

    xmObjects = None

    # Grab all objects within 3 arcsec of this one.
    xmList = []
    catalogueName = 'tcs_transient_objects'
    message, xmObjects = coneSearchHTM(ra, dec, coneSearchRadius, catalogueName, queryType = FULL, conn = connection, django = True)

    # The crossmatch Objects xmObjects are a list of two entry lists. The first entry in each row is the separaion.
    # The second entry is the catalogue row dictionary listing the relevant crossmatch.

    if xmObjects:
        numberOfMatches = len(xmObjects)
        # Add the objects into a list of dicts that have consistent names for all catalogues

        for xm in xmObjects:
            sys.stderr.write("\n%s\n" % str(xm))
            # Add to the list all object ids except the current one (which will have
            # a separation of zero).
            if xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]] != candidate:
                xmList.append({'xmseparation': xm[0], 'xmid': xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]})
                xmid = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]
                xmra = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][1]]
                xmdec = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][2]]
                xmName = xm[1]["ps1_designation"]
                if not xmName:
                    xmName = xm[1]["local_designation"]
                if not xmName:
                    xmName = xmid

                xmRecurrences = []
                xmDetections = TcsTransientReobservations.objects.filter(transient_object_id = xmid)
                for xmDet in xmDetections:
                    xmRecurrences.append({"RA": xmDet.ra_psf, "DEC": xmDet.dec_psf})
                # Add the recurrence from the objects table!
                xmRecurrences.append({"RA": xmra, "DEC": xmdec})


                xmrecurrencePlotData, xmrecurrencePlotLabels, xmaverageObjectCoords, xmrmsScatter = getRecurrenceDataForPlotting(xmRecurrences, ra, dec, secRA = xmra, secDEC = xmdec, secId = xmid, secName = xmName, objectColour = 23)
                recurrencePlotData += xmrecurrencePlotData
                recurrencePlotLabels += xmrecurrencePlotLabels
                averageObjectCoords += xmaverageObjectCoords
                rmsScatter += xmrmsScatter

    recurrenceData = [recurrencePlotData, recurrencePlotLabels, averageObjectCoords, rmsScatter]
    return recurrenceData



















import socket
# 2019-08-07 KWS added sendMessage to send a message to a listening daemon. This might be for:
#                * getting a PS name
#                * registering on TNS
#                * doing a minor planet check.

def sendMessage(host, port, message):
    '''
    Send a message to a listener.
    '''

    response = None

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.sendall(message.encode('utf-8'))
        response = s.recv(1024).decode('utf-8')
        s.close()
    except socket.error as e:
        sys.stderr.write("Cannot connect. Maybe the service has not been started or is listening on a different port.\n")
        sys.stderr.write("%s" % str(e))

    return response

def filterGetParameters(request, queryFilter, prefix = ''):
    """filterGetParameters.

    Args:
        request:
        queryFilter:
        prefix:
    """
    # When we're querying for tcs_transient_objects we probably have lots of
    # common things to pull out.  Hence this helper function. Prefix can
    # be used if we need to refer the parameters remotely, but note that it
    # applies to all parameters at the moment.

    # 2020-10-13 KWS Add ability to do greater than, greater than or equal to,
    #                less than, less than or equal to.
    import datetime
    flagDate = None
    flagParameter = 'followup_flag_date'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        flagParameter = 'followup_flag_date' + suffix
        flagDate = request.GET.get(flagParameter)
        if flagDate is not None:
            break

    try:
        flagDate = datetime.datetime.strptime(flagDate, "%Y-%m-%d").date()
    except ValueError as e:
        flagDate = None
    except TypeError as e:
        flagDate = None

    if flagDate:
        queryFilter[prefix + flagParameter] = flagDate

    # 2020-10-13 KWS Added rb_pix (i.e. RB factor)
    zooniverseScore = None
    zooniverseScoreParameter = 'zooniverse_score'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        zooniverseScoreParameter = 'zooniverse_score' + suffix
        zooniverseScore = request.GET.get(zooniverseScoreParameter)
        if zooniverseScore is not None:
            break

    try:
        zooniverseScore = float(zooniverseScore)
    except ValueError as e:
        zooniverseScore = None
    except TypeError as e:
        zooniverseScore = None

    if zooniverseScore:
        queryFilter[prefix + zooniverseScoreParameter] = zooniverseScore

    # 2020-10-13 KWS Added rb_pix (i.e. RB factor)
    rbPix = None
    rbPixParameter = 'rb_pix'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        rbPixParameter = 'rb_pix' + suffix
        rbPix = request.GET.get(rbPixParameter)
        if rbPix is not None:
            break

    try:
        rbPix = float(rbPix)
    except ValueError as e:
        rbPix = None
    except TypeError as e:
        rbPix = None

    if rbPix:
        queryFilter[rbPixParameter] = rbPix

    # 2020-12-11 KWS Added rb_cat (i.e. catalogue RB factor)
    rbCat = None
    rbCatParameter = 'rb_cat'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        rbCatParameter = 'rb_cat' + suffix
        rbCat = request.GET.get(rbCatParameter)
        if rbCat is not None:
            break

    try:
        rbCat = float(rbCat)
    except ValueError as e:
        rbCat = None
    except TypeError as e:
        rbCat = None

    if rbCat:
        queryFilter[rbCatParameter] = rbCat

    # 2020-12-11 KWS Added earliest mag & latest mag
    latestMag = None
    latestMagParameter = 'latest_mag'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        latestMagParameter = 'latest_mag' + suffix
        latestMag = request.GET.get(latestMagParameter)
        if latestMag is not None:
            break

    try:
        latestMag = float(latestMag)
    except ValueError as e:
        latestMag = None
    except TypeError as e:
        latestMag = None

    if latestMag:
        queryFilter[latestMagParameter] = latestMag


    # 2020-12-11 KWS Added earliest mag & earliest mag
    earliestMag = None
    earliestMagParameter = 'earliest_mag'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        earliestMagParameter = 'earliest_mag' + suffix
        earliestMag = request.GET.get(earliestMagParameter)
        if earliestMag is not None:
            break

    try:
        earliestMag = float(earliestMag)
    except ValueError as e:
        earliestMag = None
    except TypeError as e:
        earliestMag = None

    if earliestMag:
        queryFilter[earliestMagParameter] = earliestMag


    # 2020-12-11 KWS Added earliest mag & earliest mag
    earliestMJD = None
    earliestMJDParameter = 'earliest_mjd'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        earliestMJDParameter = 'earliest_mjd' + suffix
        earliestMJD = request.GET.get(earliestMJDParameter)
        if earliestMJD is not None:
            break

    try:
        earliestMJD = float(earliestMJD)
    except ValueError as e:
        earliestMJD = None
    except TypeError as e:
        earliestMJD = None

    if earliestMJD:
        queryFilter[earliestMJDParameter] = earliestMJD


    # 2020-12-11 KWS Added latest mag & latest mag
    latestMJD = None
    latestMJDParameter = 'latest_mjd'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        latestMJDParameter = 'latest_mjd' + suffix
        latestMJD = request.GET.get(latestMJDParameter)
        if latestMJD is not None:
            break

    try:
        latestMJD = float(latestMJD)
    except ValueError as e:
        latestMJD = None
    except TypeError as e:
        latestMJD = None

    if latestMJD:
        queryFilter[latestMJDParameter] = latestMJD


    # 2020-12-11 KWS Added earliest mag & earliest mag
    otherDesignation = None
    otherDesignationParameter = 'other_designation'
    for suffix in ['__startswith', '__endswith', '']:
        otherDesignationParameter = 'other_designation' + suffix
        otherDesignation = request.GET.get(otherDesignationParameter)
        if otherDesignation is not None:
            break

    if otherDesignation:
        queryFilter[otherDesignationParameter] = otherDesignation


    objectType = None
    objectType = request.GET.get('object_classification')
    try:
        objectType = int(objectType)
    except ValueError as e:
        objectType = None
    except TypeError as e:
        objectType = None

    if objectType:
        queryFilter[prefix + 'object_classification'] = objectType

    sherlockType = None
    sherlockType = request.GET.get('sherlockClassification')

    if sherlockType:
        queryFilter[prefix + 'sherlockClassification'] = sherlockType

    specType = None
    specType = request.GET.get('observation_status')

    if specType:
        queryFilter[prefix + 'observation_status'] = specType

    xt = None
    xtParameter = 'xt'

    for suffix in ['__isnull', '']:
        xtParameter = 'xt' + suffix
        xt = request.GET.get(xtParameter)
        if xt is not None:
            break

    try:
        xt = int(xt)
    except ValueError as e:
        xt = None
    except TypeError as e:
        xt = None

    if xt:
        queryFilter[prefix + xtParameter] = xt

    return queryFilter

def getDjangoTables2ImageTemplate(imageType):
    IMAGE_TEMPLATE = """<img id="stampimages" src="{{ MEDIA_URL }}images/data/{{ dbname }}/{{ record.tcs_images_id.whole_mjd }}/{{ record.tcs_images_id.%s }}.jpeg" alt="triplet" title="{{ record.tcs_images_id.pss_filename }}" onerror="this.src='{{ STATIC_URL }}images/image_not_available.jpeg';" height="200" />""" % imageType
    return IMAGE_TEMPLATE




"""
Django settings for atlas project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
from dotenv import load_dotenv

# import forcephot

load_dotenv(override=True)

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
PATHPREFIX = os.environ.get('WSGI_PREFIX')

SITE_ID = 1

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('DJANGO_SECRET_KEY')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

TEMPLATE_DEBUG = False

ALLOWED_HOSTS = ['*']

ROOT_URLCONF = 'atlas.urls'

WSGI_APPLICATION = 'atlas.wsgi.application'

# 2016-11-07 KWS Fixed the authentication issues by setting cookie names
CSRF_COOKIE_NAME = 'csrf_' + os.environ.get('DJANGO_MYSQL_DBNAME')
SESSION_COOKIE_NAME = 'session_' + os.environ.get('DJANGO_MYSQL_DBNAME')

# 2017-10-03 KWS Had to add this setting because of SSL proxy.
CSRF_TRUSTED_ORIGINS = ['star.pst.qub.ac.uk']

CSRF_FAILURE_VIEW = 'atlas.views.csrf_failure'

LOGIN_REDIRECT_URL = 'home'
LOGIN_URL = 'login'
LOGOUT_URL = 'logout'

# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': os.environ.get('DJANGO_MYSQL_DBNAME'),
        'USER': os.environ.get('DJANGO_MYSQL_DBUSER'),
        'PASSWORD': os.environ.get('DJANGO_MYSQL_DBPASS'),
        'HOST': os.environ.get('DJANGO_MYSQL_DBHOST'),
        'PORT': int(os.environ.get('DJANGO_MYSQL_DBPORT')),
    }
}


DAEMONS = {
    'tns': {
        'host': os.environ.get('DJANGO_TNS_DAEMON_SERVER'),
        'port': int(os.environ.get('DJANGO_TNS_DAEMON_PORT')),
        'test': False
    },
    'mpc': {
        'host': os.environ.get('DJANGO_MPC_DAEMON_SERVER'),
        'port': int(os.environ.get('DJANGO_MPC_DAEMON_PORT')),
        'test': False
    }
}


LANGUAGE_CODE = 'en-gb'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True



# 2021-05-06 KWS New settings means that if we edit a static file we MUST rerun the collectstatic
#                code to deploy the modified file.

STATIC_URL = PATHPREFIX + '/static/'

# STATICFILES_DIRS tells collectstatic where MY static files are.
STATICFILES_DIRS = (
  os.path.join(BASE_DIR, 'site_media'),
)

STATIC_ROOT = os.path.join(BASE_DIR, 'static')

# MEDIA is DIFFERENT from static files.  It's serving up files from a mounted filesystem for example.
# These files are NOT covered by collectstatic, but the webserver DOES need to know where they are!
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = PATHPREFIX + '/media/'

# All below copied from PS1 web app.

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            # insert your TEMPLATE_DIRS here
            os.path.join(BASE_DIR, 'templates'),

        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.template.context_processors.request",
            ],
        },
    },
]

MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
]

ROOT_URLCONF = 'atlas.urls'

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.messages',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    'atlas',
    'django_tables2',
]



from setup_database_utils import *

def main(argv = None):
   """main.

   Args:
       argv:
   """
   # This code will NOT use MySQLdb to connect to the database.  This is to preserve the
   # ability to run each SQL script individually.  However, we will need to capture the
   # output of each script.

   if argv is None:
      argv = sys.argv

   if len(argv) != 4:
      sys.exit("Usage: refresh_django_views.py <username> <password> <schema name>")

   user = argv[1]
   password = argv[2]
   databaseName = argv[3]

   if newSchemaAndViews(user, password, databaseName, createNewSchema = False) > 0:
      print("Something went wrong.  Terminating program.")
      return 1

   return 0


# ###########################################################################################
#                                         Main hook
# ###########################################################################################


if __name__=="__main__":
    main()




# 2021-03-11 KWS Added the old SqlPaginator code from python2 here and converted to python3.
#                (Not available from PyPI for python3.)
import logging

from math import ceil

from django.db import connection
from django.core.paginator import Page

from django.core.paginator import EmptyPage, PageNotAnInteger

import sqlparse
from sqlparse import sql
from sqlparse import tokens

logger = logging.getLogger(__name__)


class SqlPaginator(object):

    def __init__(self, initial_sql, model, order_by='id', page=1,
                count=None, per_page=10, direction='asc'):

        # if these are not an int then fail
        self.page_num = int(page)
        self.per_page = int(per_page)
        try:
            self._count = int(count)
        except TypeError:
            self._count = None

        # need the model class to do some sql validation
        self.model = model

        self._num_pages = None
        self.orphans = 0
        self._initial_raw_sql = initial_sql
        self.initial_sql = initial_sql
        self.object_list = []

        self.allow_empty_first_page = True

        # resolve the model fields into their names
        self._model_fields = [f.name for f in model._meta.fields]

        self._db_table = self.model._meta.db_table

        # validate sql fields
        if order_by not in self._model_fields:
            raise ValueError("%s is not a valid column in %s" % (order_by, self._db_table))
        
        # sometimes its really a foreign key, so get the real colum name
        order_by = self.model._meta.get_field(order_by).column

        self.order_by = order_by

        self.direction = direction
        if direction.lower() not in ['asc', 'desc']:
            self.direction = 'asc'

        # order_by queries work differently when using select distinct queries
        # maybe i should use a sql parser ?
        self._tsql = '%(sql)s order by "%(order_by)s" %(direction)s limit %(limit)d offset %(offset)d'

        # get the token list from the query, there will be only one
        tlist = sqlparse.parse(initial_sql)[0]
        from_token = tlist.token_next_match(0, tokens.Keyword, "FROM")

        select_token = tlist.token_next_match(0, tokens.DML, "Select")

        found = False
        for t in tlist.tokens_between(select_token, from_token, exclude_end=True):
            if t.value.lower().find(order_by.lower()) > -1:
                found = True
                break

        if not found:
              tlist.insert_before(from_token, sql.Token(tokens.DML, ",%s " % order_by))

        self.initial_sql = tlist.to_unicode()

        # dict to resolve the sql template with
        self.d = {'sql': self.initial_sql,
                  'order_by': order_by,
                  'offset': int(page - 1) * self.per_page,
                  'limit': self.per_page,
                  'direction': direction,
                 }

        self._sql = self._tsql % self.d

    def get_sql(self):
        return self._sql
    sql = property(get_sql)

    def _get_count(self):
        if self._count is None:
            cursor = connection.cursor()
            sql = "select count(distinct(%s)) from (%s) as q" % (self.model._meta.pk.name, self._initial_raw_sql)
            cursor.execute(sql)
            rows = cursor.fetchall()
            count = int(rows[0][0])
            self._count = count
        return self._count
    count = property(_get_count)

    def _get_num_pages(self):
        if self._num_pages is None:
            if self.count == 0:
                self._num_pages = 0
            else:
                hits = max(1, self.count, self.orphans)
                self._num_pages = int(ceil(hits / float(self.per_page)))

        return self._num_pages
    num_pages = property(_get_num_pages)

    def validate_number(self, number):
        "Validates the given 1-based page number."
        try:
            number = int(number)
        except (TypeError, ValueError):
            raise PageNotAnInteger('That page number is not an integer')
        if number < 1:
            raise EmptyPage('That page number is less than 1')
        if number > self.num_pages:
            if number == 1 and self.allow_empty_first_page:
                pass
            else:
                raise EmptyPage('That page contains no results')
        return number

    def page(self, number, order_by=None, direction=None):
        number = self.validate_number(number)

        if not order_by and order_by not in self._model_fields:
            order_by = self.order_by

        if not direction or direction.lower() not in ['asc', 'desc']:
            direction = self.direction

        self.d.update({'offset': (number - 1) * self.per_page,
                       'order_by': order_by,
                       'direction': direction,
                       })

        logger.debug('count: %d' % self.count)
        logger.debug('num_pages: %d' % self.num_pages)

        sql = self._tsql % self.d
        logger.debug("sql: %s" % sql)

        self.object_list = list(self.model.objects.raw(sql))

        return Page(self.object_list, number, self)

    def _get_page_range(self):
        """
        Returns a 1-based range of pages for iterating through within
        a template for loop.
        """
        return list(range(1, self.num_pages + 1))
    page_range = property(_get_page_range)



"""Get table columns to convert into Avro format.

Usage:
  %s <database> <hostname> <tables>...
  %s (-h | --help)
  %s --version

Options:
  -h --help              Show this screen.
  --version              Show version.
  --flagdate=<flagdate>  Flag date threshold beyond which we will select objects [default: 20170920].
  --ddc                  Assume the DDC schema.
  --truncate             Truncate the database tables. Default is NOT to truncate.

"""
import sys
__doc__ = __doc__ % (sys.argv[0], sys.argv[0], sys.argv[0])
from docopt import docopt
from gkutils.commonutils import cleanOptions, Struct, dbConnect

sqlToAvro = {
    "bigint": "long",
    "int": "int",
    "double": "double",
    "float": "float",
    "tinyint": "int",
    "smallint": "int",
    "datetime": "something",
    "varchar": "string"}


def getTableInfo(conn, schema, table):
    """
    Get the ATLAS table we want to convert into avro format

    :param conn: database connection
    :param table: table name

    """
    import MySQLdb

    try:
        cursor = conn.cursor(MySQLdb.cursors.DictCursor)

        cursor.execute ("""
            select column_name,
                     data_type, 
                   is_nullable
              from information_schema.columns
             where table_schema = %s COLLATE utf8_general_ci
               and table_name = %s COLLATE utf8_general_ci ;
        """, (schema, table))

        resultSet = cursor.fetchall ()
        cursor.close ()

    except MySQLdb.Error as e:
        sys.stderr.write("Error %d: %s\n" % (e.args[0], e.args[1]))
        sys.exit (1)

    return resultSet



def main(argv = None):
    opts = docopt(__doc__, version='0.1')
    opts = cleanOptions(opts)

    # Use utils.Struct to convert the dict into an object for compatibility with old optparse code.
    options = Struct(**opts)

    conn = dbConnect(options.hostname, 'kws', '', options.database)

    if options.tables:
        for row in options.tables:
            data = getTableInfo(conn, options.database, row)
            for column in data:
                if column['is_nullable'] == 'YES':
                    print('{"name": "%s", "type": ["%s", "null"], "default": "null", "doc": ""},' % (column['column_name'], sqlToAvro[column['data_type']]))
                else:
                    print('{"name": "%s", "type": "%s", "doc": ""},' % (column['column_name'], sqlToAvro[column['data_type']]))

    conn.close()



if __name__ == '__main__':
    main()



from django.conf.urls import url
from django.urls import path, include
from django.contrib import admin
from atlas import views
from django.conf import settings
from django.conf.urls.static import static

admin.autodiscover()


urlpatterns = [
    # 2016-02-24 KWS Introduced the Django Admin URL
    path('admin/', admin.site.urls),
    # 2016-02-26 KWS Add the authentication URLs
    url(r'^accounts/login/', views.login, name="login"),
    url(r'^accounts/logout/', views.logout, name="logout"),
    url(r'^accounts/auth/', views.authView, name="auth"),
    url(r'^accounts/loggedin/', views.loggedin, name="loggedin"),
    url(r'^accounts/invalid/', views.invalidLogin, name="invalid"),

    url(r'^$', views.homepage, name='home'),

    # 2019-09-28 KWS New error page.
    url(r'^error/$', views.errorpage, name='error'),

    # 2017-06-16 KWS New ddc detections
#    url(r'^lightcurve/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurveplain, name='lightcurveplain'),
    url(r'^lightcurve/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurveplainddc, name='lightcurveplain'),
    url(r'^lightcurveforced/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurveforcedplain, name='lightcurveforcedplain'),
    url(r'^atel/(?P<tcs_transient_objects_id>\d+)/$', views.atel, name='atel'),

    # 2017-06-16 KWS New ddc detections
#    url(r'^candidate/(?P<atlas_diff_objects_id>\d+)/$', views.candidate, name='candidate'),
    url(r'^candidate/(?P<atlas_diff_objects_id>\d+)/$', views.candidateddc, {'template_name':'candidate_plotly.html'}, name='candidate'),
    url(r'^candidate_bs/(?P<atlas_diff_objects_id>\d+)/$', views.candidateddc, {'template_name':'candidate_plotly.html'}, name='candidate_bs'),
    url(r'^candidate_old/(?P<atlas_diff_objects_id>\d+)/$', views.candidateddc, {'template_name':'candidate.html'}, name='candidate_old'),

    url(r'^userlist_atel_discovery/(?P<userDefinedListNumber>\d+)/$', views.atelsDiscovery, name='ateldiscovery'),
    url(r'^userlist_atel_fast/(?P<userDefinedListNumber>\d+)/$', views.atelsFast, name='atelfast'),
    url(r'^userlist_visibility/(?P<userDefinedListNumber>\d+)/$', views.visibility, name='visibility'),
    url(r'^userlist_iobserve/(?P<userDefinedListNumber>\d+)/$', views.iobserve, name='iobserve'),


    url(r'^externalcrossmatches/$', views.displayExternalCrossmatches, name='externalcrossmatches'),

    url(r'^followup/(?P<listNumber>\d+)/$', views.followupList, name='followup'),
    url(r'^followuptxt/(?P<listNumber>\d+)/$', views.followuptxt, name='followuptxt'),
    url(r'^followupsubsettxt/(?P<listNumber>\d+)/$', views.followupsubsettxt, name='followupsubsettxt'),
    url(r'^pesstosummary/$', views.pesstosummary, name='pesstosummary'),
#    url(r'^pesstorecurrences/$', views.pesstorecurrences, name='pesstorecurrences'),
    url(r'^pesstorecurrences/$', views.pesstorecurrencesddc, name='pesstorecurrences'),

    url(r'^summarycsv/(?P<listNumber>\d+)/$', views.summarycsv, name='summarycsv'),

#    url(r'^followup2/(?P<listNumber>\d+)/$', views.followupList2, name='followup2'),

    url(r'^followup3/(?P<listNumber>\d+)/$', views.followupList3, name='followup3'),

    # Experiment!
    url(r'^followup_bypass_django_tables/(?P<listNumber>\d+)/$', views.followup_bypass_django_tables, name='followup_bypass_django_tables'),

    url(r'^userlist/$', views.userDefinedListDefinitions, name='userdefinedlistdefs'),
    url(r'^userlist/(?P<userDefinedListNumber>\d+)/$', views.userDefinedLists, name='userdefinedlists'),

# 2016-06-15 KWS Added quickview URLs.
    url(r'^followup_quickview/(?P<listNumber>\d+)/$', views.followupQuickView, name='followupquickview'),
    url(r'^followup_quickview_bs/(?P<listNumber>\d+)/$', views.followupQuickViewBootstrapPlotly, name='followupquickviewbootstrapplotly'),
    url(r'^followup_quickview/$', views.followupAllQuickView, name='followupallquickview'),
    url(r'^userlist_quickview/(?P<userDefinedListNumber>\d+)/$', views.userDefinedListsQuickview, name='userdefinedlistsquickview'),

# 2016-08-27 KWS Search Results URL
    url(r'^searchresults/$', views.searchResults, name='searchresults'),
    url(r'^snejson/$', views.jsonSNe, name='snejson'),
]


if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL,
                          document_root=settings.MEDIA_ROOT)



from django.conf.urls import url
from django.urls import path, include
from django.contrib import admin
from psdb import views
from django.conf import settings
from django.conf.urls.static import static

admin.autodiscover()

urlpatterns = [
    path('admin/', admin.site.urls),

    # 2016-07-07 KWS Add the authentication URLs
    url(r'^accounts/login/', views.login, name="login"),
    url(r'^accounts/logout/', views.logout, name="logout"),
    url(r'^accounts/auth/', views.authView, name="auth"),
    url(r'^accounts/loggedin/', views.loggedin, name="loggedin"),
    url(r'^accounts/invalid/', views.invalidLogin, name="invalid"),

    url(r'^$', views.redirectedHomepage, name='redirectedhome'),
    url(r'^psdb/$', views.homepage2, name='home2'),

    # 2020-09-30 KWS New error page.
    url(r'^error/$', views.errorpage, name='error'),

#    url(r'^psdb/test/$', views.homepage2, name='home2'),

    url(r'^psdb/temporary/$', views.temporaryHomepage),
    url(r'^psdb/dss2/(?P<tcs_transient_objects_id>\d+)/$', views.dss2),
    url(r'^psdb/lightcurves/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurves),
# 2011-07-06 KWS Add TEXT version of the light curve
    url(r'^psdb/lightcurve/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurveplain),
    url(r'^psdb/lightcurveforced/(?P<tcs_transient_objects_id>\d+)/$', views.lightcurveforcedplain),
# 2013-02-08 KWS Add TEXT version of the colour plot
    url(r'^psdb/colour/(?P<tcs_transient_objects_id>\d+)/$', views.colourdataplain),
# 2012-07-18 KWS Experimental Flot lightcurves
    url(r'^psdb/candidate/(?P<tcs_transient_objects_id>\d+)/$', views.candidateflot, name="candidate"),
    url(r'^psdb/astronote/(?P<tcs_transient_objects_id>\d+)/$', views.astronote, name='astronote'),

# NEW format Followup Lists
    url(r'^psdb/followup/$', views.followupAllNew, name="followuplistall"),

    url(r'^psdb/followup/(?P<listNumber>\d+)/$', views.followupListNew, name="followuplist"),
    url(r'^psdb/followup_quickview/(?P<listNumber>\d+)/$', views.followupQuickView, name='followupquickview'),
    url(r'^psdb/followup_quickview/$', views.followupAllQuickView),

# Geojson view of the supernovae
    url(r'^snejson/$', views.jsonSNe, name='snejson'),

# PUBLIC release pages
    url(r'^psdb/public/$', views.followupAllPublic, name="followuplistallpublic"),
    url(r'^psdb/public_quickview/$', views.followupAllPublicQuickView, name="public_quickview"),
    url(r'^psdb/public_textonly/$', views.followupAllPublicTextOnly),

# 2011-04-14 KWS Add new User Defined Lists URL
    url(r'^psdb/userlist/$', views.userDefinedListDefinitions, name='userdefinedlistdefs'),
    url(r'^psdb/userlist/(?P<userDefinedListNumber>\d+)/$', views.userDefinedLists, name='userdefinedlists'),
    url(r'^psdb/userlist_quickview/(?P<userDefinedListNumber>\d+)/$', views.userDefinedListsQuickview, name='userdefinedlistsquickview'),

    url(r'^psdb/userlistcat/(?P<userDefinedListNumber>\d+)/$', views.obsCatalogue),
    url(r'^psdb/userlistwiki/(?P<userDefinedListNumber>\d+)/$', views.obsMediaWiki),
    url(r'^psdb/userlist_atel_discovery/(?P<userDefinedListNumber>\d+)/$', views.atelsDiscovery),
    url(r'^psdb/userlist_gcn/(?P<userDefinedListNumber>\d+)/$', views.gcn),
    url(r'^psdb/userlist_gcn_latex/(?P<userDefinedListNumber>\d+)/$', views.gcnlatex),

# 2014-07-23 KWS Added New CSV lists
    url(r'^psdb/followupcsv/(?P<listNumber>\d+)/$', views.followupCsv),

# 2011-05-08 KWS Add new Reports directory
    url(r'^psdb/reports/$', views.reportspage),

    url(r'^psdb/crossmatchcfatoipp/$', views.displayCfAtoIPPCrossmatches),
    url(r'^psdb/crossmatchipptocfa/$', views.displayIPPtoCfACrossmatches),

# 2013-10-21 KWS Add new External Crossmatch page
    url(r'^psdb/externalcrossmatches/$', views.displayExternalCrossmatches),

# 2016-08-27 KWS Search Results URL
    url(r'^searchresults/$', views.searchResults, name='searchresults'),
]

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL,
                          document_root=settings.MEDIA_ROOT)



# 2012-07-18 KWS This file contains all the relevant custom queries needed to build lightcurves using 'flot'.
#                They are created as custom queries for SPEED. It is also necessary to build new custom
#                objects in dbviews. Note that the code in this file relies on Django objects.  Other
#                code has been abstracted out to commonqueries so that it can be called by external scripts.

# 2012-09-26 KWS Add numpy so we can do some simple max/min calculations on the lightcurve data
import numpy as n
from .commonqueries import *
import json
from math import log10

GRIZY_COLOURS = {"g": 0, "r": 1, "i": 2, "z": 3, "y": 4}

MIN_Y_AXIS_MAG_DIFF = 3.0

# 2018-01-11 KWS We need to mask out bad photometry from faulty amplifiers.
MLO_BAD_AMP_x = [0, 659]
MLO_BAD_AMP_y = [0, 5279]

FORCED_PHOT_DAYS_BEFORE_DETECTION = 30

def getColourPlotData(g, r, i, z, y):
    """Collect the colour info from input filter data for plotting"""
    from gkutils.commonutils import getColour, getColourStats
    import json

    colourPlotLimits = {}
    coloursJSON = []

    grColour = []
    riColour = []
    izColour = []
    colourPlotLabels = [{'label': 'g-r', 'color': 0, 'display': True},
                        {'label': 'r-i', 'color': 1, 'display': True},
                        {'label': 'i-z', 'color': 2, 'display': True}]
    meangr = None
    meanri = None
    meaniz = None

    grEvolution = None
    riEvolution = None
    izEvolution = None

    if g and r:
        grColour = getColour(g, r, FILTER_RECURRENCE_PERIOD_GR)
        if grColour and len(grColour) > 1:
            meangr, grEvolution = getColourStats(grColour)
            colourPlotLabels[0]["label"] += (' (avg = %4.2f mag,' % meangr) + (' trend = %5.3f mag/d)' % grEvolution)
    if r and i:
        riColour = getColour(r, i, FILTER_RECURRENCE_PERIOD_RI)
        if riColour and len(riColour) > 1:
            meanri, riEvolution = getColourStats(riColour)
            colourPlotLabels[1]["label"] += (' (avg = %4.2f mag,' % meanri) + (' trend = %5.3f mag/d)' % riEvolution)
    if i and z:
        izColour = getColour(i, z, FILTER_RECURRENCE_PERIOD_IZ)
        if izColour and len(izColour) > 1:
            meaniz, izEvolution = getColourStats(izColour)
            colourPlotLabels[2]["label"] += (' (avg = %4.2f mag,' % meaniz) + (' trend = %5.3f mag/d)' % izEvolution)

    xMin, xMax, yMin, yMax = getLimitsFromLCData([grColour, riColour, izColour])
    colourPlotLimits["xmin"] = xMin
    colourPlotLimits["xmax"] = xMax
    colourPlotLimits["ymin"] = yMin
    colourPlotLimits["ymax"] = yMax
    colours = [grColour, riColour, izColour]
    colourPlotLabelsJSON = [json.dumps(colourPlotLabels[0]), json.dumps(colourPlotLabels[1]), json.dumps(colourPlotLabels[2])]

    return colours, colourPlotLimits, colourPlotLabelsJSON

# 2015-11-26 KWS New LC code for ATLAS
# 2017-11-30 KWS Changed code to call organiseLCPlot (like the forced photometry). This
#                greatly simplifies this method and adds the option of having non-detections.

def getLCData(candidate, conn = None, getFollowupData = False, limits = LIMITS, ddc = False, getNonDetections = True):
    """getLCData.

    Args:
        candidate:
        conn:
        getFollowupData:
        limits:
        ddc:
        getNonDetections:
    """
    import json
    from gkutils.commonutils import getCurrentMJD
    import sys

    dataList = []

    from .dbviews import CustomLCPoints, CustomFollowupLCData, CustomLCBlanks

    #lcData, recurrences = getLightcurvePoints(candidate, djangoRawObject = CustomLCPoints, conn = conn, lcQuery = lcQuery)
    if ddc:
        p, recurrences = getLightcurvePoints(candidate, djangoRawObject = CustomLCPoints, lcQuery=LC_POINTS_QUERY_ATLAS_DDC + filterWhereClauseddc(FILTERS), conn = conn)
    else:
        p, recurrences = getLightcurvePoints(candidate, djangoRawObject = CustomLCPoints, conn = conn)

    if getNonDetections:
        if ddc:
            b, blanks, lastNonDetection = getNonDetectionsUsingATLASFootprint(recurrences, djangoRawObject = CustomLCBlanks, conn = conn, ndQuery=ATLAS_METADATADDC, filterWhereClause = filterWhereClauseddc, catalogueName = 'atlas_metadataddc')
        else:
            b, blanks, lastNonDetection = getNonDetectionsUsingATLASFootprint(recurrences, djangoRawObject = CustomLCBlanks, conn = conn)
    else:
        b = []

    lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = organiseLCPlot(p, b, displayNegativeMags = True)

    return lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits


def getAllLCData(candidate, applyFudge = True, getColours = True, conn = None, getFollowupData = False, limits = LIMITS):
    """getAllLCData.

    Args:
        candidate:
        applyFudge:
        getColours:
        conn:
        getFollowupData:
        limits:
    """

    # 2012-07-18 KWS Add in custom queries to get lightcurve data.
    # 2013-02-12 KWS Added conn object, so that we can grab a remote database lightcurve.

    # 2014-07-09 KWS This code is a bit of a mess.  We need to make it more filter agnostic.
    #                In the meantime, to get it to work quickly, I've just added any new
    #                filters to the original list (currently grizywxBV).  We can have up to
    #                20 filters before I need to alter the lightcurve code.  The LAST value
    #                in the list is always the combined data, so ALWAYS refer to it by index
    #                of -1.

    import json
    from gkutils.commonutils import getCurrentMJD
    import sys

    # NOTE: At this point we have the data for ALL filters in the 6th row of lcData.
    #       We can use this to set the max/min limits in x (and y) if necessary.
    #       The javascript code is currently not using them, but it should. We should
    #       have checkboxes for whole survey lifetime, detection limits, etc...
    #       We just have to decide how to send the max/min data to the lightcurve code.
    #       Probably need a new varable so we don't get confused.

    # Data is currently [g, r, i, z, y, w, x, B, V, all]

    dataList = []

    # 2013-02-03 KWS Dynamically set the django raw query object here instead of in the query code.
    #                This makes the query code portable to non django MySQL queries.

    from .dbviews import CustomLCPoints, CustomFollowupLCData

    lcData = getLightcurvePoints(candidate, applyFudge = applyFudge, djangoRawObject = CustomLCPoints, conn = conn)

    if lcData and lcData[-1]:
        dataList.append(lcData[-1])
    lcPointsJSON = [json.dumps(lcData[0]), json.dumps(lcData[1]), json.dumps(lcData[2]), json.dumps(lcData[3]), json.dumps(lcData[4]), json.dumps(lcData[5]), json.dumps(lcData[6]), json.dumps(lcData[7]), json.dumps(lcData[8])]

    # For the colour plots (later) list the griz data separately
    g = lcData[0]
    r = lcData[1]
    i = lcData[2]
    z = lcData[3]
    y = lcData[4]
    w = lcData[5]
    x = lcData[6]
    B = lcData[7]
    V = lcData[8]

    grizyArray = n.array(lcData[-1])
    discoveryDate = n.min(grizyArray[:,0])

    lcData = getLightcurveBlanks(candidate, djangoRawObject = CustomLCPoints, limits = limits)
    if lcData and lcData[-1]:
        dataList.append(lcData[-1])

    gblank = lcData[0]
    rblank = lcData[1]
    iblank = lcData[2]
    zblank = lcData[3]
    yblank = lcData[4]
    wblank = lcData[5]
    xblank = lcData[6]
    Bblank = lcData[7]
    Vblank = lcData[8]

    lcBlanksJSON = [json.dumps(lcData[0]), json.dumps(lcData[1]), json.dumps(lcData[2]), json.dumps(lcData[3]), json.dumps(lcData[4]), json.dumps(lcData[5]), json.dumps(lcData[6]), json.dumps(lcData[7]), json.dumps(lcData[8])]

    lcData = getLightcurveNonDetections(candidate, djangoRawObject = CustomLCPoints, limits = limits)
    if lcData and lcData[-1]:
        dataList.append(lcData[-1])

    gnon = lcData[0]
    rnon = lcData[1]
    inon = lcData[2]
    znon = lcData[3]
    ynon = lcData[4]
    wnon = lcData[5]
    xnon = lcData[6]
    Bnon = lcData[7]
    Vnon = lcData[8]

    lcNonDetectionsJSON = [json.dumps(lcData[0]), json.dumps(lcData[1]), json.dumps(lcData[2]), json.dumps(lcData[3]), json.dumps(lcData[4]), json.dumps(lcData[5]), json.dumps(lcData[6]), json.dumps(lcData[7]), json.dumps(lcData[8])]

    # plotLabels MUST be the same length as the max number of series plotted.  In this case, there should be number_of_filters
    # x 3 data series.  Additionally plotLabels also defines the colour of each series.  Why is this done? So that we can append
    # other lightcurves from followup photometry tables (e.g. Liverpool Telescope photometry).  The data is used directly by
    # javascript so it needs to be in json format.

    # 2014-07-09 KWS Added bool(list) as the value of the display parameter.  This means only labels for lists that actually
    #                exist will get displayed.  Need also to pick out the non-detections.

    plotLabels = [json.dumps({'label':'g', 'color': 0, 'display': bool(g) or bool(gblank) or bool(gnon)}),
                  json.dumps({'label':'r', 'color': 1, 'display': bool(r) or bool(rblank) or bool(rnon)}),
                  json.dumps({'label':'i', 'color': 2, 'display': bool(i) or bool(iblank) or bool(inon)}),
                  json.dumps({'label':'z', 'color': 3, 'display': bool(z) or bool(zblank) or bool(znon)}),
                  json.dumps({'label':'y', 'color': 4, 'display': bool(y) or bool(yblank) or bool(ynon)}),
                  json.dumps({'label':'w', 'color': 5, 'display': bool(w) or bool(wblank) or bool(wnon)}),
                  json.dumps({'label':'x', 'color': 6, 'display': bool(x) or bool(xblank) or bool(xnon)}),
                  json.dumps({'label':'B', 'color': 7, 'display': bool(B) or bool(Bblank) or bool(Bnon)}),
                  json.dumps({'label':'V', 'color': 8, 'display': bool(V) or bool(Vblank) or bool(Vnon)}),
                  json.dumps({'label':'g blanks', 'color': 0, 'display': False}),
                  json.dumps({'label':'r blanks', 'color': 1, 'display': False}),
                  json.dumps({'label':'i blanks', 'color': 2, 'display': False}),
                  json.dumps({'label':'z blanks', 'color': 3, 'display': False}),
                  json.dumps({'label':'y blanks', 'color': 4, 'display': False}),
                  json.dumps({'label':'w blanks', 'color': 5, 'display': False}),
                  json.dumps({'label':'x blanks', 'color': 6, 'display': False}),
                  json.dumps({'label':'B blanks', 'color': 7, 'display': False}),
                  json.dumps({'label':'V blanks', 'color': 8, 'display': False}),
                  json.dumps({'label':'g non det', 'color': 0, 'display': False}),
                  json.dumps({'label':'r non det', 'color': 1, 'display': False}),
                  json.dumps({'label':'i non det', 'color': 2, 'display': False}),
                  json.dumps({'label':'z non det', 'color': 3, 'display': False}),
                  json.dumps({'label':'y non det', 'color': 4, 'display': False}),
                  json.dumps({'label':'w non det', 'color': 5, 'display': False}),
                  json.dumps({'label':'x non det', 'color': 6, 'display': False}),
                  json.dumps({'label':'B non det', 'color': 7, 'display': False}),
                  json.dumps({'label':'V non det', 'color': 8, 'display': False})]

    followupDetectionData = []
    followupDetectionDataBlanks = []
    if getFollowupData:
        followupFilterData, followupFilterDataBlanks, followupFullList, followupFullListBlanks = getFollowupPhotometry(candidate, djangoRawObject = CustomFollowupLCData, conn = conn)
        dataList.append(followupFullList)
        dataList.append(followupFullListBlanks)
        # get all the keys, and assign labels, colours and display flag to them

        followupPlotColours = {}
        initialFollowupPlotColour = 15

        for filter, filterdata in list(followupFilterData.items()):
            if filter not in followupPlotColours:
                followupPlotColours[filter] = initialFollowupPlotColour
                initialFollowupPlotColour += 1

            plotLabels.append(json.dumps({'label':filter, 'color': followupPlotColours[filter], 'display': True}))

            followupDetectionData.append(followupFilterData[filter])

        for filter, filterdata in list(followupFilterDataBlanks.items()):
            if filter not in followupPlotColours:
                followupPlotColours[filter] = initialFollowupPlotColour
                initialFollowupPlotColour += 1

            plotLabels.append(json.dumps({'label':filter, 'color': followupPlotColours[filter], 'display': False}))

            followupDetectionDataBlanks.append(followupFilterDataBlanks[filter])


    xMin, xMax, yMin, yMax = getLimitsFromLCData(dataList)

    today = getCurrentMJD()
    plotLimits = {}
    plotLimits["xmin"] = xMin
    plotLimits["xmax"] = xMax
    plotLimits["ymin"] = yMin
    plotLimits["ymax"] = yMax
    plotLimits["discoveryDate"] = discoveryDate
    plotLimits["today"] = today

    colourPlotData = []
    colourPlotLimits = []
    colourPlotLabels = []

    if getColours:
        colourPlotData, colourPlotLimits, colourPlotLabels = getColourPlotData(g, r, i, z, y)

    return lcPointsJSON, lcBlanksJSON, lcNonDetectionsJSON, followupDetectionData, followupDetectionDataBlanks, plotLabels, plotLimits, colourPlotData, colourPlotLimits, colourPlotLabels



def getForcedLCData(candidate, getColours = True, conn = None, limits = LIMITS):
    """Get Forced Photometry plot data, including colours if necessary"""

    from .dbviews import CustomForcedLCData
    import json
    from gkutils.commonutils import getCurrentMJD
    import sys

    dataList = []

    forcedDetectionData = []
    forcedDetectionDataBlanks = []
    plotLabels = []

    forcedDetectionDataFlux = []
    plotLabelsFlux = []

    forcedFilterData, forcedFilterDataBlanks, forcedFullList, forcedFullListBlanks, forcedFilterDataFlux, forcedFullListFlux = getForcedPhotometry(candidate, djangoRawObject = CustomForcedLCData, conn = conn, limits = limits)

    # If we don't get any data no point proceeding.
    if not forcedFilterData or not forcedFilterDataFlux:
        return []

    dataList.append(forcedFullList)
    dataList.append(forcedFullListBlanks)
    # get all the keys, and assign labels, colours and display flag to them

    for filter, filterdata in list(forcedFilterData.items()):
        plotLabels.append(json.dumps({'label':filter, 'color': GRIZY_COLOURS[filter], 'display': True}))
        forcedDetectionData.append(forcedFilterData[filter])

    for filter, filterdata in list(forcedFilterDataBlanks.items()):
        plotLabels.append(json.dumps({'label':filter, 'color': GRIZY_COLOURS[filter], 'display': False}))
        forcedDetectionDataBlanks.append(forcedFilterDataBlanks[filter])

    for filter, filterdata in list(forcedFilterDataFlux.items()):
        plotLabelsFlux.append(json.dumps({'label':filter, 'color': GRIZY_COLOURS[filter], 'display': True}))
        forcedDetectionDataFlux.append(forcedFilterDataFlux[filter])

    # Grab the forced phot limits and use those to set the x limits
    grizyArray = n.array(forcedFullList)
    discoveryDate = n.min(grizyArray[:,0])

    xMin, xMax, yMin, yMax = getLimitsFromLCData(dataList)
    today = getCurrentMJD()
    plotLimits = {}
    plotLimits["xmin"] = xMin
    plotLimits["xmax"] = xMax
    plotLimits["ymin"] = yMin
    plotLimits["ymax"] = yMax
    plotLimits["discoveryDate"] = discoveryDate
    plotLimits["today"] = today

    # Grab the flux limits
    grizyArray = n.array(forcedFullListFlux)
    discoveryDate = n.min(grizyArray[:,0])

    xMin, xMax, yMin, yMax = getLimitsFromLCData([forcedFullListFlux])
    fluxLimits = {}
    fluxLimits["xmin"] = xMin
    fluxLimits["xmax"] = xMax
    fluxLimits["ymin"] = yMin
    fluxLimits["ymax"] = yMax
    fluxLimits["discoveryDate"] = discoveryDate
    fluxLimits["today"] = today

    colourPlotData = []
    colourPlotLimits = []
    colourPlotLabels = []

    if getColours:
        try:
            g = forcedFilterData["g"]
        except KeyError as e:
            g = []
        try:
            r = forcedFilterData["r"]
        except KeyError as e:
            r = []
        try:
            i = forcedFilterData["i"]
        except KeyError as e:
            i = []
        try:
            z = forcedFilterData["z"]
        except KeyError as e:
            z = []
        try:
            y = forcedFilterData["y"]
        except KeyError as e:
            y = []
        
        colourPlotData, colourPlotLimits, colourPlotLabels = getColourPlotData(g, r, i, z, y)

    return forcedDetectionData, forcedDetectionDataBlanks, plotLabels, plotLimits, forcedDetectionDataFlux, plotLabelsFlux, fluxLimits, colourPlotData, colourPlotLimits, colourPlotLabels



def getRecurrenceDataForPlotting(recurrences, primRA, primDEC, secRA = None, secDEC = None, secId = None, secName = None, objectColour = 0):
    """Grab the recurrence data in JSON format for plotting with Flot"""

    import json, math

    averageObjectCoords, rmsScatter = getRecurrenceData(recurrences)

    recurrenceData = []

    for row in recurrences:
        recurrenceData.append([(primRA - row["RA"]) * math.cos(math.radians(row["DEC"])) * -3600.0, (primDEC - row["DEC"]) * 3600.0])

    # The recurrence plot data will contain:
    # * The recurrences (deltas)
    # * The primary detection
    # * The average coordinates

    # The mean coordinates are to be displayed at the origin, so just dump [0,0] as the third series

    if secRA and secDEC:
        # This is a neighbouring detection
        # 2013-08-26 KWS Flipped east-west by multiplying x by -1.
        primaryDetection = [[(primRA - secRA) * math.cos(math.radians(primDEC)) * -3600.0, (primDEC - secDEC) * 3600.0]]
        # Add an anchor tag so that we can click on the legend to go to the new object.
        # This should really be done inside the javascript itself, but it's hard to know
        # which label to add a tag to, hence it's done here.
        # 2018-09-13 KWS Because this code is also called from new followup pages, force URL to go up
        #                two directories.
        objectName = '<a href="../../candidate/' + str(secId) + '/">' + str(secName) + '</a>'
        meanCoordsLabel = 'mean coords'
        meanCoordsDisplay = False
    else:
        primaryDetection = [[0, 0]]
        objectName = '1st detection'
        meanCoordsLabel = 'mean coords (rms = %4.2f)' % rmsScatter
        meanCoordsDisplay = True


    recurrencePlotData = [json.dumps(recurrenceData),
                          json.dumps(primaryDetection),
                          json.dumps([[(primRA - averageObjectCoords["RA"]) * math.cos(math.radians(primDEC)) * -3600.0, (primDEC - averageObjectCoords["DEC"]) * 3600.0]])]


    recurrencePlotLabels = [json.dumps({'label': 'Recurrences', 'color': objectColour, 'display': False}),
                            json.dumps({'label': objectName, 'color': objectColour+1, 'display': True}),
                            json.dumps({'label': meanCoordsLabel, 'color': objectColour+2, 'display': meanCoordsDisplay})]

    return recurrencePlotData, recurrencePlotLabels, [averageObjectCoords], [rmsScatter]


def organiseLCPlot(lcData, lcBlanks, displayNegativeMags = False, daysBeforeDetection = 0):
    """organiseLCPlot.

    Args:
        lcData:
        lcBlanks:
        displayNegativeMags:
        daysBeforeDetection:
    """
    import sys
    from gkutils.commonutils import getCurrentMJD

    dataList = []
    if lcData and lcData[-1]:
        dataList.append(lcData[-1])
    if lcBlanks and lcBlanks[-1]:
        dataList.append(lcBlanks[-1])

    lcPointsJSON = [json.dumps(p) for p in lcData[:-1]]
    lcBlanksJSON = [json.dumps(p) for p in lcBlanks[:-1]]

    plotLabelsJSON = []
    for idx,filter in enumerate(FILTERS):
        exec ('%s = lcData[%d]' % (filter, idx))
        plotLabelsJSON.append(json.dumps({'label':filter, 'color': idx, 'display': bool(eval(filter))}))

    if displayNegativeMags:
        for idx,filter in enumerate(FILTERS):
            exec ('%s = lcData[%d]' % ('n' + filter, idx + len(FILTERS)))
            plotLabelsJSON.append(json.dumps({'label':'-' + filter, 'color': idx, 'display': bool(eval(('n' + filter)))}))

    if lcBlanks:
        for idx,filter in enumerate(FILTERS):
            exec ('%s = lcBlanks[%d]' % (filter, idx))
            plotLabelsJSON.append(json.dumps({'label':filter, 'color': idx, 'display': False}))

    grizyArray = n.array(lcData[-1])
    discoveryDate = n.min(grizyArray[:,0]) - daysBeforeDetection

    xMin, xMax, yMin, yMax = getLimitsFromLCData(dataList)


    # Create a lightcurve with at least a 4 mag range on the y axis

    if abs(yMax[-1] - yMin[-1]) < MIN_Y_AXIS_MAG_DIFF:
        yAvg = (yMax[-1] + yMin[-1])/2.0
        yMax[-1] = yAvg + MIN_Y_AXIS_MAG_DIFF/2.0
        yMin[-1] = yAvg - MIN_Y_AXIS_MAG_DIFF/2.0

    # Add 10% headroom to the ymin & ymax values
    yMin[-1] = yMin[-1] - 0.2 * (abs(yMax[-1] - yMin[-1]))
    yMax[-1] = yMax[-1] + 0.2 * (abs(yMax[-1] - yMin[-1]))


    today = getCurrentMJD()
    plotLimits = {}
    plotLimits["xmin"] = xMin
    plotLimits["xmax"] = xMax
    plotLimits["ymin"] = yMin
    plotLimits["ymax"] = yMax
    plotLimits["discoveryDate"] = discoveryDate
    plotLimits["today"] = today

    sys.stderr.write("\nPLOT LIMITS\n")
    sys.stderr.write("\n%s\n" % str(plotLimits))

    return lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits


def organiseAtlasForcedPhotometryForPlotting(forcedPhotometry):
    """organiseAtlasForcedPhotometryForPlotting.

    Args:
        forcedPhotometry:
    """
    # Go through the filters and create subarrays of each filter
    filtersList = []
    filtersListBlanks = []
    # Create a bunch of empty lists for each filter
    for row in FILTERS:
        exec("%s = []" % row)
    for row in FILTERS:
        exec("%sblanks = []" % row)

    fullList = []
    fullListBlanks = []

    # 2018-01-10 KWS The Forced Photometry code calculates down to 1sigma by default but
    #                calculates 3sigma limiting magnitudes.
    #                These are maybe useful for detailed analysis, but on the lightcurves
    #                the 1sigma "detections" are distracting.
    #                So - new rule: if snrdet < snrlimit, and detection snr < snrlimit,
    #                recalculate new limiting mag at current snrlimit.
    for row in forcedPhotometry:
        if row.limiting_mag and row.mag is not None and row.mjd_obs is not None:
            fullListBlanks.append([row.mjd_obs, row.mag])
        else:
            if row.mag is not None and row.dm is not None and row.mjd_obs is not None:
                if row.snr >= row.snrlimit:
                    # Are we in a bad amplifier? If so, create another non-detection.
                    if row.x <= MLO_BAD_AMP_x[1] and row.y <= MLO_BAD_AMP_y[1] and '01a' in row.expname.obs and row.mjd_obs < 58718:
                        newLimitingMag = row.mag - 2.5 * log10(row.dpeak * row.snrlimit / row.peakfit)
                        fullListBlanks.append([row.mjd_obs, newLimitingMag])
                    else:
                        # We probed deeper (to 1sigma by default), but we only want to show detections above the set SNR limit.
                        fullList.append([row.mjd_obs, row.mag, row.dm])
                else:
                    # We recorded a mag, but now calculate a limiting mag instead.
                    newLimitingMag = row.mag - 2.5 * log10(row.dpeak * row.snrlimit / row.peakfit)
                    fullListBlanks.append([row.mjd_obs, newLimitingMag])
        if row.filter in FILTERS:
            if row.limiting_mag and row.mag is not None and row.mjd_obs is not None:
                eval(row.filter + 'blanks').append([row.mjd_obs, row.mag])
            else:
                if row.mag is not None and row.dm is not None and row.mjd_obs is not None:
                    if row.snr >= row.snrlimit:
                        # Are we in a bad amplifier? If so, create another non-detection.
                        if row.x <= MLO_BAD_AMP_x[1] and row.y <= MLO_BAD_AMP_y[1] and '01a' in row.expname.obs and row.mjd_obs < 58718:
                            newLimitingMag = row.mag - 2.5 * log10(row.dpeak * row.snrlimit / row.peakfit)
                            eval(row.filter + 'blanks').append([row.mjd_obs, newLimitingMag]) 
                        else:
                            eval(row.filter).append([row.mjd_obs, row.mag, row.dm])
                    else:
                        newLimitingMag = row.mag - 2.5 * log10(row.dpeak * row.snrlimit / row.peakfit)
                        eval(row.filter + 'blanks').append([row.mjd_obs, newLimitingMag]) 

    for f in FILTERS:
        filtersList.append(eval(f))
        filtersListBlanks.append(eval(f + 'blanks'))

    filtersList.append(fullList)
    filtersListBlanks.append(fullListBlanks)

    # 2019-07-01 KWS Occasionally, because the object happens to be either SNR < 3 or in the dodgy MLO chip,
    #                the fullList will be empty.  In this case abort! Don't bother attempting to render the
    #                lightcurve.
    if len(fullList) == 0:
        return [],[],[],[]


    lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = organiseLCPlot(filtersList, filtersListBlanks, daysBeforeDetection = FORCED_PHOT_DAYS_BEFORE_DETECTION)
    return lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits

# 2018-09-20 KWS New code to show the flux in uJy.
#                f[uJy] = peak*major*minor/etime * dex(-0.4*(ZP+apfit-23.9))

def organiseAtlasForcedFluxPhotometryForPlotting(forcedPhotometry):
    """organiseAtlasForcedFluxPhotometryForPlotting.

    Args:
        forcedPhotometry:
    """
    # Go through the filters and create subarrays of each filter
    filtersList = []
    filtersListBlanks = []
    # Create a bunch of empty lists for each filter
    for row in FILTERS:
        exec("%s = []" % row)
    for row in FILTERS:
        exec("%sblanks = []" % row)

    fullList = []

    #factor=exp(-0.4*log(10)*(zp+apfit-23.9))
    #print "%.1f %.1f" % ($1*$3*$4/dt*factor, $2*$3*$4/dt*factor)

    for row in forcedPhotometry:
        if row.peakfit is not None \
            and row.dpeak is not None \
            and row.mjd_obs is not None \
            and row.major is not None \
            and row.minor is not None \
            and (row.expname.apfit is not None or row.apfit is not None) \
            and not (row.x <= MLO_BAD_AMP_x[1] and row.y <= MLO_BAD_AMP_y[1] and '01a' in row.expname.obs and row.mjd_obs < 58718):

            # 2018-09-25 KWS Use local table apfit and fixed value of exp time (30s) if no
            #                no value for apfit and texp in the metadata file (true if we are
            #                in the OLD DDT based database).
            apfit = row.expname.apfit
            if apfit is None:
                apfit = row.apfit
            exptime = row.expname.texp
            if exptime is None:
                exptime = 30.0

            factor = 10**(-0.4*(row.zp+apfit-23.9))
            uJy = row.peakfit*row.major*row.minor/exptime*factor
            duJy = row.dpeak*row.major*row.minor/exptime*factor

            fullList.append([row.mjd_obs, uJy, duJy])

            eval(row.filter).append([row.mjd_obs, uJy, duJy])

    for f in FILTERS:
        filtersList.append(eval(f))

    filtersList.append(fullList)

    lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = ([],[],[],[])
    if filtersList[-1]:
        lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = organiseLCPlot(filtersList, [], daysBeforeDetection = FORCED_PHOT_DAYS_BEFORE_DETECTION)

    return lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits


# 2019-09-20 KWS New code to show the flux in uJy when calculated using John's
#                code.

def organiseAtlasForcedFluxPhotometryForPlottingJT(forcedPhotometryJT):
    """organiseAtlasForcedFluxPhotometryForPlottingJT.

    Args:
        forcedPhotometryJT:
    """
    import sys
    # Go through the filters and create subarrays of each filter
    filtersList = []
    # Create a bunch of empty lists for each filter
    for row in FILTERS:
        exec("%s = []" % row)
    for row in FILTERS:
        exec("%sblanks = []" % row)

    fullList = []

    for row in forcedPhotometryJT:
        fullList.append([row.mjd, row.ujy, row.dujy])

        eval(row.f).append([row.mjd, row.ujy, row.dujy])

    for f in FILTERS:
        filtersList.append(eval(f))

    filtersList.append(fullList)

    lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = ([],[],[],[])
    if filtersList[-1]:
        sys.stderr.write("\nOrganising stacked diff flux\n")
        lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits = organiseLCPlot(filtersList, [], daysBeforeDetection = FORCED_PHOT_DAYS_BEFORE_DETECTION)

    return lcPointsJSON, lcBlanksJSON, plotLabelsJSON, plotLimits
